package de.duehl.mp3.player;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.IOException;
import javazoom.jl.decoder.JavaLayerException;
import javazoom.jl.player.AudioDevice;
import javazoom.jl.player.FactoryRegistry;
import javazoom.jl.player.advanced.AdvancedPlayer;

/**
 * Ein Player für MP3-Dateien.
 *
 * @author David J. Barnes und Michael Kölling, überarbeitet von Christian Dühl
 * @version 31.07.2011, 13.10.2023
 *
 * Siehe auch: https://falconbyte.net/blog-java-mp3.php, hieß ursprünglich "MusikPlayer".
 */

public class MP3Player {

    /** Der JavaZoom-Player. */
    private AdvancedPlayer player;

    /** Konstruktor. */
    public MP3Player() {
        player = null;
    }

    /** Spielt die erste halbe Sekunde eines Musikstücks an. */
    public void playFirstPart(String filename) {
        try {
            preparePlayer(filename);
            player.play(500);
        }
        catch (JavaLayerException exception) {
            reportProblem(filename);
        }
        finally {
            killPlayer();
        }
    }

    /** Spielt das Musikstück in einem eigenen Tread ab. */
    public void playMP3InOwnThread(String filename) {
        try {
            tryToStartPlayInOwnThread(filename);
        }
        catch (Exception exception) {
            reportProblem(filename);
        }
    }

    private void tryToStartPlayInOwnThread(String filename) {
        preparePlayer(filename);
        new Thread(() -> startPlay(filename)).start();
    }

    private void startPlay(String filename) {
        try {
            player.play(5000);
        }
        catch (JavaLayerException exception) {
            reportProblem(filename);
        }
        finally {
            killPlayer();
        }
    }

    /** Beendet das Abspielen des aktuell gespielten Musikstücks. */
    public void stop() {
        killPlayer();
    }

    private void preparePlayer(String filename) {
        try {
            InputStream stream = createInputStream(filename);
            player = new AdvancedPlayer(stream, createAudioDevice());
        }
        catch (IOException exception) {
            reportProblem(filename);
            killPlayer();
        }
        catch (JavaLayerException exception) {
            reportProblem(filename);
            killPlayer();
        }
    }

    private InputStream createInputStream(String dateiname) throws IOException {
        return new BufferedInputStream(new FileInputStream(dateiname));
    }

    private AudioDevice createAudioDevice() throws JavaLayerException {
        return FactoryRegistry.systemRegistry().createAudioDevice();
    }

    private void killPlayer() {
        synchronized (this) {
            if (player != null) {
                player.stop();
                player = null;
            }
        }
    }

    private void reportProblem(String filename) {
        System.out.println("Es gab ein Problem beim Abspielen von: " + filename);
    }

}
