package de.duehl.vocabulary.japanese.data.symbol;

import java.util.ArrayList;
import java.util.List;

import de.duehl.basics.text.Text;

/**
 * Diese Klasse stellt die Zeichen der Schrift Hiragana (in UTF-Zeichen) dar.
 *
 * Sie beinhaltet auch Methoden rund darum, um zu fragen, ob eine Zeichenkette ausschließlich
 * aus diesen Zeichen besteht oder wo das erste bzw. letzte dieser Zeichen zu finden ist.
 *
 * @version 1.01     2024-10-02
 * @author Christian Dühl
 */

public enum Hiragana {

    A("あ", "a"),
    I("い", "i"),
    U("う", "u"),
    E("え", "e"),
    O("お", "o"),

    KA("か", "ka"),
    KI("き", "ki"),
    KU("く", "ku"),
    KE("け", "ke"),
    KO("こ", "ko"),

    SA("さ", "sa"),
    SHI("し", "shi"),
    SU("す", "su"),
    SE("せ", "se"),
    SO("そ", "so"),

    TA("た", "ta"),
    CHI("ち", "chi"),
    TSU("つ", "tsu"),
    TE("て", "te"),
    TO("と", "to"),

    NA("な", "na"),
    NI("に", "ni"),
    NU("ぬ", "nu"),
    NE("ね", "ne"),
    NO("の", "no"),

    HA("は", "ha"),
    HI("ひ", "hi"),
    FU("ふ", "fu"),
    HE("へ", "he"),
    HO("ほ", "ho"),

    MA("ま", "ma"),
    MI("み", "mi"),
    MU("む", "mu"),
    ME("め", "me"),
    MO("も", "mo"),

    YA("や", "ya"),
    YU("ゆ", "yu"),
    YO("よ", "yo"),

    SMALL_YA("ゃ", "kleines ya"),
    SMALL_YU("ゅ", "kleines yu"),
    SMALL_YO("ょ", "kleines yo"),

    RA("ら", "ra"),
    RI("り", "ri"),
    RU("る", "ru"),
    RE("れ", "re"),
    RO("ろ", "ro"),

    WA("わ", "wa"),
    WO("を", "wo"),

    N("ん", "n"),

    GA("が", "ga"),
    GI("ぎ", "gi"),
    GU("ぐ", "gu"),
    GE("げ", "ge"),
    GO("ご", "go"),

    ZA("ざ", "za"),
    JI1("じ", "ji"),
    ZU1("ず", "zu"),
    ZE("ぜ", "ze"),
    ZO("ぞ", "zo"),

    DA("だ", "da"),
    JI2("ぢ", "ji"),
    ZU2("づ", "zu"),
    DE("で", "de"),
    DO("ど", "do"),

    BA("ば", "ba"),
    BI("び", "bi"),
    BU("ぶ", "bu"),
    BE("べ", "be"),
    BO("ぼ", "bo"),

    PA("ぱ", "pa"),
    PI("ぴ", "pi"),
    PU("ぷ", "pu"),
    PE("ぺ", "pe"),
    PO("ぽ", "po"),

    KYA("きゃ", "kya"),
    KYU("きゅ", "kyu"),
    KYO("きょ", "kyo"),

    SHA("しゃ", "sha"),
    SHU("しゅ", "shu"),
    SHO("しょ", "sho"),

    CHA("ちゃ", "cha"),
    CHU("ちゅ", "chu"),
    CHO("ちょ", "cho"),

    NYA("にゃ", "nya"),
    NYU("にゅ", "nyu"),
    NYO("にょ", "nyo"),

    HYA("ひゃ", "hya"),
    HYU("ひゅ", "hyu"),
    HYO("ひょ", "hyo"),

    MYA("みゃ", "mya"),
    MYU("みゅ", "myu"),
    MYO("みょ", "myo"),

    RYA("りゃ", "rya"),
    RYU("りゅ", "ryu"),
    RYO("りょ", "ryo"),

    GYA("ぎゃ", "gya"),
    GYU("ぎゅ", "gyu"),
    GYO("ぎょ", "gyo"),

    JA1("じゃ", "ja"),
    JU1("じゅ", "ju"),
    JO1("じょ", "jo"),

    JA2("ぢゃ", "ja"),
    JU2("ぢゅ", "ju"),
    JO2("ぢょ", "jo"),

    BYA("びゃ", "bya"),
    BYU("びゅ", "byu"),
    BYO("びょ", "byo"),

    PYA("ぴゃ", "pya"),
    PYU("ぴゅ", "pyu"),
    PYO("ぴょ", "pyo"),

    LEHNWORT_WI("ゔぃ", "wi"),
    LEHNWORT_FA("ふぉ", "fa"),
    LEHNWORT_TI("てぃ", "ti"),
    LEHNWORT_DU("どぅ", "du"),
    LEHNWORT_WE("うぇ", "we"),
    LEHNWORT_FO("ふぉ", "fo"),

    SMALL_TSU("っ", "kleines tsu"),
    ;

    /** Das Zeichen in UTF-8 Darstellung. */
    private final String character;

    /** Die Hepburn-Darstellung in lateinischen Buchstaben. */
    private final String hepburn;

    /**
     * Konstruktor.
     *
     * @param character
     *            Das Zeichen in UTF-8 Darstellung.
     * @param hepburn
     *            Die Hepburn-Darstellung in lateinischen Buchstaben.
     */
    private Hiragana(String character, String hepburn) {
        this.character = character;
        this.hepburn = hepburn;
    }

    /** Getter für das Zeichen in UTF-8 Darstellung. */
    public String getCharacter() {
        return character;
    }

    /** Getter für die Hepburn-Darstellung in lateinischen Buchstaben. */
    public String getHepburn() {
        return hepburn;
    }

    /** Gibt das Hiragana-Zeichen als Kana-Objekt zurück. */
    public Kana toKana() {
        Kana kana = new Kana(character, hepburn);
        return kana;
    }

    /** Gibt eine Liste mit den Zeichen der Schrift Hiragana (in UTF-Zeichen) zurück. */
    public static List<String> getAllHiraganaAsStringList() {
        List<String> hiraganaList = new ArrayList<>();

        for (Hiragana hiragana : Hiragana.values()) {
            hiraganaList.add(hiragana.getCharacter());
        }

        return hiraganaList;
    }

    /** Gibt an, ob in dem Text ein Hiragana-Zeichen vorkommt. */
    public static boolean containsHiragana(String text) {
        return Text.contains(text, getAllHiraganaAsStringList());
    }

    /** Gibt an, ob der Text ausschließlich aus Hiragana-Zeichen besteht. */
    public static boolean containsOnlyHiragana(String text) {
        if (text.isEmpty()) {
            return false;
        }
        else {
            List<String> hiragana = getAllHiraganaAsStringList();
            for (String textCharacter : Text.textToCharactersList(text)) {
                if (!hiragana.contains(textCharacter)) {
                    return false;
                }
            }

            return true;
        }
    }

    /**
     * Gibt den Index des ersten Hiragana-Zeichen im Text zurück. Gibt es kein solches, wird -1
     * zurückgegeben.
     */
    public static int firstIndexOfHiragana(String text) {
        List<String> hiragana = getAllHiraganaAsStringList();
        List<String> textCharacters = Text.textToCharactersList(text);

        for (int index = 0; index < textCharacters.size(); ++index) {
            String textCharacter = textCharacters.get(index);
            if (hiragana.contains(textCharacter )) {
                return index;
            }
        }

        return -1;
    }

    /**
     * Gibt den Index des letzten Hiragana-Zeichen im Text zurück. Gibt es kein solches, wird -1
     * zurückgegeben.
     */
    public static int lastIndexOfHiragana(String text) {
        List<String> hiragana = getAllHiraganaAsStringList();
        List<String> textCharacters = Text.textToCharactersList(text);

        for (int index = textCharacters.size() - 1; index >= 0; --index) {
            String textCharacter = textCharacters.get(index);
            if (hiragana.contains(textCharacter )) {
                return index;
            }
        }

        return -1;
    }

}
