package de.duehl.vocabulary.japanese.data.symbol;

import java.util.ArrayList;
import java.util.List;

import de.duehl.basics.collections.CollectionsHelper;
import de.duehl.basics.text.Text;

/**
 * Diese Klasse stellt die Zeichen der Schrift Kanji (in UTF-Zeichen) dar, die ich bislang gelernt
 * habe. Da es davon über 50.000 Stück gibt, wird diese Klasse niemals vollständig sein.
 *
 * Ich hoffe aber, sie enthält irgendwann die 2.136 常用漢字 (Jōyō-Kanji), vgl.
 * https://de.wikipedia.org/wiki/J%C5%8Dy%C5%8D-Kanji.
 *
 * Die Klasse beinhaltet auch Methoden rund darum, um zu fragen, ob eine Zeichenkette
 * ausschließlich aus diesen Zeichen besteht oder wo das erste bzw. letzte dieser Zeichen zu finden
 * ist.
 *
 * Möchte man an die Übersetzungen, so kann man nach dem Wort über normalKanjiHiraganaWord oder
 * normalHiraganaWord suchen, das muss man nicht hier auch noch vorhalten.
 *
 * https://www-kanshudo-com.translate.goog/kanji/%E6%B0%97?_x_tr_sl=en&_x_tr_tl=de&_x_tr_hl=de&_x_tr_pto=rq
 *
 * @version 1.01     2024-10-04
 * @author Christian Dühl
 */

public enum Kanji {

    FISCH("魚", "Fisch", "魚", "さかな", 290, 112,
            CollectionsHelper.buildListFrom("GYO"),
            CollectionsHelper.buildListFrom("sakana", "ue")
            ),
    FLEISCH("肉", "Fleisch", "肉", "にく", 223, 102,
            CollectionsHelper.buildListFrom("NIKU"),
            CollectionsHelper.buildListFrom()
            ),
    EI("卵", "Ei", "卵", "たまご", 1103, 228,
            CollectionsHelper.buildListFrom("RAN"),
            CollectionsHelper.buildListFrom("tamago")
            ),
    WASSER("水", "Wasser", "水", "みず", 21, 73,
            CollectionsHelper.buildListFrom("SUI"),
            CollectionsHelper.buildListFrom("mizu")
            ),
    ESSEN("食", "essen", "食べる", "たべる", 322, 116,
            CollectionsHelper.buildListFrom("SHOKU", "[JIKI]"),
            CollectionsHelper.buildListFrom("ta(beru)", "ku(u)", "ku(rau)")
            ),
    TRINKEN("飲", "trinken", "飲む", "のむ", 323, 117,
            CollectionsHelper.buildListFrom("IN"),
            CollectionsHelper.buildListFrom("no(mu)")
            ),
    GROSS("大", "groß", "大きい", "ちいさい", 26, 74,
            CollectionsHelper.buildListFrom("DAI", "TAI"),
            CollectionsHelper.buildListFrom("oo(kii)", "oo-", "oo(ini")
            ),
    KLEIN("小", "klein", "小さい", "ちいさい", 27, 74,
            CollectionsHelper.buildListFrom("SHOO"),
            CollectionsHelper.buildListFrom("chii(sai)", "ko-", "o-")
            ),
    NEU("新", "neu", "新しい", "あたらしい", 174, 95,
            CollectionsHelper.buildListFrom("SHIN"),
            CollectionsHelper.buildListFrom("atara(shii)", "ara(ta)", "nii-")
            ),
    ALT("古", "alt", "古い", "ふるい", 172, 95,
            CollectionsHelper.buildListFrom("KO"),
            CollectionsHelper.buildListFrom("furu(i)", "furu(su)")
            ),
    TAG_SONNE("日", "Tag, Sonne", "日", "にち", 5, 71,
            CollectionsHelper.buildListFrom("NICHI", "JITSU"),
            CollectionsHelper.buildListFrom("hi", "ka")
            ),
    BUCH_URSPRUNG("本", "Buch, Ursprung", "本", "ほん", 25, 74,
            CollectionsHelper.buildListFrom("HON"),
            CollectionsHelper.buildListFrom("moto")
            ),
    PERSON_MENSCH("人", "Person, Mensch", "人", "ひと", 1, 71,
            CollectionsHelper.buildListFrom("JIN", "NIN"),
            CollectionsHelper.buildListFrom("hito")
            ),
    GEHEN_FAHREN("行", "gehen, fahren, fliegen", "行く", "いく", 68, 80,
            CollectionsHelper.buildListFrom("KOU", "[AN]", "GYOU"),
            CollectionsHelper.buildListFrom("i(ku)", "yu(ku)", "okona(u)")
            ),
    ELEKTRIZITAET("電", "Elektrizität", "電車", "でんしゃ", 108, 86,
            CollectionsHelper.buildListFrom("DEN"),
            CollectionsHelper.buildListFrom()
            ),
    AUTO_WAGEN_RAD("車", "Auto, Wagen, Rad", "車", "くるま", 133, 89,
            CollectionsHelper.buildListFrom("SHA"),
            CollectionsHelper.buildListFrom("kuruma")
            ),
    FRAU("女", "Frau", "女", "おんな", 102, 85,
            CollectionsHelper.buildListFrom("JO", "NYO", "]NYOU]"),
            CollectionsHelper.buildListFrom("onna", "me")
            ),
    MANN("男", "Mann", "男", "おとこ", 101, 85,
            CollectionsHelper.buildListFrom("DAN", "NAN"),
            CollectionsHelper.buildListFrom("otoko")
            ),
    SPRECHEN("話", "sprechen, reden", "話す", "はなす", 238, 104,
            CollectionsHelper.buildListFrom("WA"),
            CollectionsHelper.buildListFrom("hanashi", "hana(su)")
            ),
    SICH_TREFFEN("会", "sich treffen", "会う", "あう", 158, 93,
            CollectionsHelper.buildListFrom("KAI", "E"),
            CollectionsHelper.buildListFrom("a(u)")
            ),
    SICH_AUSRUHEN("休", "sich ausruhen", "休む", "休", 60, 79,
            CollectionsHelper.buildListFrom("KYUU"),
            CollectionsHelper.buildListFrom("yasu(mu)", "yasu(meru)", "yasu(maru)")
            ),
    SEHEN_ANSEHEN("見", "sehen, ansehen", "見る", "みる", 63, 79,
            CollectionsHelper.buildListFrom("KEN"),
            CollectionsHelper.buildListFrom("mi(ru)", "mi(eru)", "mi(seru)")
            ),
    HIMMEL("天", "Himmel", "天気", "てんき", 141, 91,
            CollectionsHelper.buildListFrom("TEN"),
            CollectionsHelper.buildListFrom("ame", "[ama]")
            ),
    GEIST_SEELE_ENERGIE("気", "Geist, Seele, Energie", "元気", "げんき", 134, 90,
            CollectionsHelper.buildListFrom("KI", "KE"),
            CollectionsHelper.buildListFrom()
            ),
    STADT_ORTSCHAFT("町", "Stadt, Ortschaft", "町", "まち", 182, 96,
            CollectionsHelper.buildListFrom("CHOU"),
            CollectionsHelper.buildListFrom("machi")
            ),
    LERNEN("学", "lernen", "大学", "だいがく", 109, 86,
            CollectionsHelper.buildListFrom("GAKU"),
            CollectionsHelper.buildListFrom("mana(bu)")
            ),
    BLUME("花", "Blume", "花", "はな", 255, 107,
            CollectionsHelper.buildListFrom("KA"),
            CollectionsHelper.buildListFrom("hana")
            ),
    JETZT("今", "jetzt", "今", "いま", 51, 78,
            CollectionsHelper.buildListFrom("KON", "KIN"),
            CollectionsHelper.buildListFrom("ima")
            ),
    ZEIT_STUNDE("時", "Zeit, Stunde, ... Uhr", "時", "じ", 42, 76,
            CollectionsHelper.buildListFrom("JI"),
            CollectionsHelper.buildListFrom("toki")
            ),
    TEILEN_VERSTEHEN_TEIL_MINUTE("分", "teilen, Teil, Minute, verstehen", "分かる", "わかる", 38, 76,
            CollectionsHelper.buildListFrom("BUN", "FUN", "PUN"),
            CollectionsHelper.buildListFrom("wa(keru/katsu)", "wa(kareru)", "wa(karu)")
            ),
    SPITZE_ZUVOR("先", "Spitze, zuvor", "先", "さき", 50, 78,
            CollectionsHelper.buildListFrom("SEN"),
            CollectionsHelper.buildListFrom("saki")
            ),
    GEBAEREN_LEBEN("生", "gebären, leben", "生む", "うむ", 44, 77,
            CollectionsHelper.buildListFrom("SEI", "SHOO"),
            CollectionsHelper.buildListFrom("i(kiru)", "i(keru)", "i(kasu)", "u(mu)", "ha(yasu)",
                    "ha(eru)", "o(u)")
            ),
    SPRACHE("語", "Sprache", "語", "ご", 67, 80,
            CollectionsHelper.buildListFrom("GO"),
            CollectionsHelper.buildListFrom("kata(ru)", "kata(rau)")
            ),
    REGEN("雨", "Regen", "雨", "あめ", 30, 75,
            CollectionsHelper.buildListFrom("U"),
            CollectionsHelper.buildListFrom("ame", "[ama]")
            ),
    KOMMEN("来", "kommen", "来る", "くる", 69, 80,
            CollectionsHelper.buildListFrom("RAI"),
            CollectionsHelper.buildListFrom("ku(ru)", "kita(su)")
            ),
    FEUER("火", "Feuer", "火", "ひ", 20, 73,
            CollectionsHelper.buildListFrom("KA"),
            CollectionsHelper.buildListFrom("hi", "[ho]")
            ),
    BAUM_HOLZ("木", "Baum, Holz", "木", "き", 22, 74,
            CollectionsHelper.buildListFrom("BOKU", "MOKU"),
            CollectionsHelper.buildListFrom("ki", "[ka]")
            ),
    WENIG("少", "wenig", "少し", "すこし", 144, 91,
            CollectionsHelper.buildListFrom("SHOU"),
            CollectionsHelper.buildListFrom("suko(shi)", "suko(nai)")
            ),
    VIEL_ZAHLREICH("多", "viel,  zahlreich", "多い", "おおい", 229, 103,
            CollectionsHelper.buildListFrom("TA"),
            CollectionsHelper.buildListFrom("oo(i)")
            ),
    WAS("何", "was", "何", "なに", 392, 126,
            CollectionsHelper.buildListFrom("KA"),
            CollectionsHelper.buildListFrom("nani", "[nan]")
            ),
    HOEREN_FRAGEN("聞", "hören, fragen", "聞く", "きく", 64, 80,
            CollectionsHelper.buildListFrom("BUN", "MON"),
            CollectionsHelper.buildListFrom("ki(ku)", "ki(koru)")
            ),
    KAUFEN("買", "kaufen", "買う", "かう", 241, 105,
            CollectionsHelper.buildListFrom("BAI"),
            CollectionsHelper.buildListFrom("ka(u)")
            ),
    HOCH_TEUER("高", "hoch, teuer", "高い", "たかい", 190, 98,
            CollectionsHelper.buildListFrom("KOUI"),
            CollectionsHelper.buildListFrom("taka(i)", "taka", "taka(maru)", "taka(meru)")
            ),
    BILLIG_SICHER("安", "billig, (sicher)", "安い", "やすい", 105, 85,
            CollectionsHelper.buildListFrom("AN"),
            CollectionsHelper.buildListFrom("yasu(i)")
            ),
    KIND("子", "Kind", "子供", "こども", 103, 85,
            CollectionsHelper.buildListFrom("SHI", "SU"),
            CollectionsHelper.buildListFrom("ko")
            ),
    WEISS("白", "Weiß", "白", "しろ", 205, 100,
            CollectionsHelper.buildListFrom("HAKU", "BYAKU"),
            CollectionsHelper.buildListFrom("shiro(i)", "shiro")
            ),
    LADEN_GESCHAEFT("店", "Laden, Geschäft", "店", "みせ", 168, 94,
            CollectionsHelper.buildListFrom("TEN"),
            CollectionsHelper.buildListFrom("mise")
            ),
    MOND_MONAT("月", "Mond, Monat", "月", "つき", 17, 73,
            CollectionsHelper.buildListFrom("GETSU", "GATSU"),
            CollectionsHelper.buildListFrom("tsuki")
            ),
    FUENF_5("五", "fünf, 5", "五", "ご", 7, 71,
            CollectionsHelper.buildListFrom("GO"),
            CollectionsHelper.buildListFrom("itsu(tsu)", "itsu-")
            ),
    MITTE_INNEN("中", "Mitte, innen", "中", "なか", 28, 74,
            CollectionsHelper.buildListFrom("CHUU", "-CHUU", "-JUU"),
            CollectionsHelper.buildListFrom("naka")
            ),
    MITTAG("午", "Mittag", "午後", "ごご", 49, 77,
            CollectionsHelper.buildListFrom("GO"),
            CollectionsHelper.buildListFrom()
            ),
    EINS_1("一", "eins, 1", "一", "いち", 2, 71,
            CollectionsHelper.buildListFrom("ICHI", "ITSU"),
            CollectionsHelper.buildListFrom("hito(tsu)", "hito-")
            ),
    ZWEI_2("二", "zwei, 2", "二", "に", 3, 71,
            CollectionsHelper.buildListFrom("NI"),
            CollectionsHelper.buildListFrom("futa(tsu)", "futa-")
            ),
    DREI_3("三", "drei, 3", "三", "ご", 4, 71,
            CollectionsHelper.buildListFrom("SAN"),
            CollectionsHelper.buildListFrom("mit(tsu)", "mi(tsu)", "mi-")
            ),
    VIER_4("四", "vier, 4", "四", "し", 6, 71,
            CollectionsHelper.buildListFrom("SHI"),
            CollectionsHelper.buildListFrom("yot(tsu)", "yo(tsu)", "yo-", "yon")
            ),
    SECHS_6("六", "sechs, 6", "六", "ろく", 8, 72,
            CollectionsHelper.buildListFrom("ROKU"),
            CollectionsHelper.buildListFrom("mut(tsu)", "mu(tsu)", "mu-", "[mui-]")
            ),
    ;

    /** Das Kanji-Zeichen in UTF-8 Darstellung. */
    private final String character;

    /** Die deutsche Beschreibung. */
    private final String germanDescription;

    /** Das Wort in Kanji und Hiragana in der häufigsten Bedeutung. */
    private final String normalKanjiHiraganaWord;

    /** Das Wort in Hiragana in der häufigsten Bedeutung. */
    private final String normalHiraganaWord;

    /** Die Nummer des Kanji im Buch "Kanji und Kana" von Wolfgang Hadamitzky. */
    private final int numberInKanjiAndKanaBook;

    /**
     * Die Seite auf der das Kanji im Buch "Kanji und Kana" von Wolfgang Hadamitzky zu finden ist.
     */
    private final int pageNumberInKanjiAndKanaBook;

    /** Die Liste der sinojapanischen ON-Lesungen des Kanji (groß geschrieben). */
    private final List<String> onLesungen;

    /** Die Liste der rein japanischen kun-Lesungen des Kanji (klein geschrieben). */
    private final List<String> kunLesungen;

    /**
     * Konstruktor.
     *
     * @param character
     *            Das Kanji-Zeichen in UTF-8 Darstellung.
     * @param germanDescription
     *            Die deutsche Beschreibung.
     * @param normalKanjiHiraganaWord
     *            Das Wort in Kanji und Hiragana in der häufigsten Bedeutung.
     * @param normalHiraganaWord
     *            Das Wort in Hiragana in der häufigsten Bedeutung.
     * @param numberInKanjiAndKanaBook
     *            Die Nummer des Kanji im Buch "Kanji und Kana" von Wolfgang Hadamitzky.
     * @param pageNumberInKanjiAndKanaBook
     *            Die Seite auf der das Kanji im Buch "Kanji und Kana" von Wolfgang Hadamitzky zu
     *            finden ist.
     * @param onLesungen
     *            Die Liste der sinojapanischen ON-Lesungen des Kanji (groß geschrieben).
     * @param kunLesungen
     *            Die Liste der rein japanischen kun-Lesungen des Kanji (klein geschrieben).
     */
    private Kanji(String character, String germanDescription, String normalKanjiHiraganaWord,
            String normalHiraganaWord, int numberInKanjiAndKanaBook,
            int pageNumberInKanjiAndKanaBook, List<String> onLesungen, List<String> kunLesungen) {
        this.character = character;
        this.germanDescription = germanDescription;
        this.normalKanjiHiraganaWord = normalKanjiHiraganaWord;
        this.normalHiraganaWord = normalHiraganaWord;
        this.numberInKanjiAndKanaBook = numberInKanjiAndKanaBook;
        this.pageNumberInKanjiAndKanaBook = pageNumberInKanjiAndKanaBook;
        this.onLesungen = onLesungen;
        this.kunLesungen = kunLesungen;
    }

    /** Getter für das Kanji-Zeichen in UTF-8 Darstellung. */
    public String getCharacter() {
        return character;
    }

    /** Getter für die deutsche Beschreibung. */
    public String getGermanDescription() {
        return germanDescription;
    }

    /** Getter für das Wort in Kanji und Hiragana in der häufigsten Bedeutung. */
    public String getNormalKanjiHiraganaWord() {
        return normalKanjiHiraganaWord;
    }

    /** Getter für das Wort in Hiragana in der häufigsten Bedeutung. */
    public String getNormalHiraganaWord() {
        return normalHiraganaWord;
    }

    /** Getter für die Nummer des Kanji im Buch "Kanji und Kana" von Wolfgang Hadamitzky. */
    public int getNumberInKanjiAndKanaBook() {
        return numberInKanjiAndKanaBook;
    }

    /**
     * Getter für die Seite auf der das Kanji im Buch "Kanji und Kana" von Wolfgang Hadamitzky zu
     * finden ist.
     */
    public int getPageNumberInKanjiAndKanaBook() {
        return pageNumberInKanjiAndKanaBook;
    }

    /** Getter für die Liste der sinojapanischen ON-Lesungen des Kanji (groß geschrieben). */
    public List<String> getOnLesungen() {
        return onLesungen;
    }

    /** Getter für die Liste der rein japanischen kun-Lesungen des Kanji (klein geschrieben). */
    public List<String> getKunLesungen() {
        return kunLesungen;
    }

    /** Gibt eine Liste mit den Zeichen der Schrift Kanji (in UTF-Zeichen) zurück. */
    public static List<String> getAllKanjiAsStringList() {
        List<String> kanjiList = new ArrayList<>();

        for (Kanji kanji : Kanji.values()) {
            kanjiList.add(kanji.getCharacter());
        }

        return kanjiList;
    }

    /** Gibt an, ob in dem Text ein Kanji-Zeichen vorkommt. */
    public static boolean containsKanji(String text) {
        return Text.contains(text, getAllKanjiAsStringList());
    }

    /** Gibt an, ob der Text ausschließlich aus Kanji-Zeichen besteht. */
    public static boolean containsOnlyKanji(String text) {
        if (text.isEmpty()) {
            return false;
        }
        else {
            List<String> kanji = getAllKanjiAsStringList();
            for (String textCharacter : Text.textToCharactersList(text)) {
                if (!kanji.contains(textCharacter)) {
                    return false;
                }
            }

            return true;
        }
    }

    /**
     * Gibt den Index des ersten Kanji-Zeichen im Text zurück. Gibt es kein solches, wird -1
     * zurückgegeben.
     */
    public static int firstIndexOfKanji(String text) {
        List<String> kanji = getAllKanjiAsStringList();
        List<String> textCharacters = Text.textToCharactersList(text);

        for (int index = 0; index < textCharacters.size(); ++index) {
            String textCharacter = textCharacters.get(index);
            if (kanji.contains(textCharacter )) {
                return index;
            }
        }

        return -1;
    }

    /**
     * Gibt den Index des letzten Kanji-Zeichen im Text zurück. Gibt es kein solches, wird -1
     * zurückgegeben.
     */
    public static int lastIndexOfKanji(String text) {
        List<String> kanji = getAllKanjiAsStringList();
        List<String> textCharacters = Text.textToCharactersList(text);

        for (int index = textCharacters.size() - 1; index >= 0; --index) {
            String textCharacter = textCharacters.get(index);
            if (kanji.contains(textCharacter )) {
                return index;
            }
        }

        return -1;
    }

}
