package de.duehl.vocabulary.japanese.data.symbol;

import java.util.ArrayList;
import java.util.List;

import de.duehl.basics.text.Text;

/**
 * Diese Klasse stellt die Zeichen der Schrift Katakana (in UTF-Zeichen) dar.
 *
 * Sie beinhaltet auch Methoden rund darum, um zu fragen, ob eine Zeichenkette ausschließlich
 * aus diesen Zeichen besteht oder wo das erste bzw. letzte dieser Zeichen zu finden ist.
 *
 * @version 1.01     2024-10-02
 * @author Christian Dühl
 */

public enum Katakana {

    A("ア", "a"),
    I("イ", "i"),
    U("ウ", "u"),
    E("エ", "e"),
    O("オ", "o"),

    KA("カ", "ka"),
    KI("キ", "ki"),
    KU("ク", "ku"),
    KE("ケ", "ke"),
    KO("コ", "ko"),

    SA("サ", "sa"),
    SHI("シ", "shi"),
    SU("ス", "su"),
    SE("セ", "se"),
    SO("ソ", "so"),

    TA("タ", "ta"),
    CHI("チ", "chi"),
    TSU("ツ", "tsu"),
    TE("テ", "te"),
    TO("ト", "to"),

    NA("ナ", "na"),
    NI("ニ", "ni"),
    NU("ヌ", "nu"),
    NE("ネ", "ne"),
    NO("ノ", "no"),

    HA("ハ", "ha"),
    HI("ヒ", "hi"),
    FU("フ", "fu"),
    HE("ヘ", "he"),
    HO("ホ", "ho"),

    MA("マ", "ma"),
    MI("ミ", "mi"),
    MU("ム", "mu"),
    ME("メ", "me"),
    MO("モ", "mo"),

    YA("ヤ", "ya"),
    YU("ユ", "yu"),
    YO("ヨ", "yo"),

    SMALL_YA("ャ", "kleines ya"),
    SMALL_YU("ュ", "kleines yu"),
    SMALL_YO("ョ", "kleines yo"),

    RA("ラ", "ra"),
    RI("リ", "ri"),
    RU("ル", "ru"),
    RE("レ", "re"),
    RO("ロ", "ro"),

    WA("ワ", "wa"),
    WO("ヲ", "wo"),

    N("ン", "n"),

    GA("ガ", "ga"),
    GI("ギ", "gi"),
    GU("グ", "gu"),
    GE("ゲ", "ge"),
    GO("ゴ", "go"),

    ZA("ザ", "za"),
    JI1("ジ", "je"),
    ZU1("ズ", "zu"),
    ZE("ゼ", "ze"),
    ZO("ゾ", "zo"),

    DA("ダ", "da"),
    JI2("ヂ", "ji"),
    ZU2("ヅ", "zu"),
    DE("デ", "de"),
    DO("ド", "do"),

    BA("バ", "ba"),
    BI("ビ", "bi"),
    BU("ブ", "bu"),
    BE("ベ", "be"),
    BO("ボ", "bo"),

    PA("パ", "pa"),
    PI("ピ", "pi"),
    PU("プ", "pu"),
    PE("ペ", "pe"),
    PO("ポ", "po"),

    KYA("キャ", "kya"),
    KYU("キュ", "kyu"),
    KYO("キョ", "kyo"),

    SHA("シャ", "sha"),
    SHU("シュ", "shu"),
    SHO("ショ", "sho"),

    CHA("チャ", "cha"),
    CHU("チュ", "chu"),
    CHO("チョ", "cho"),

    NYA("ニャ", "nya"),
    NYU("ニュ", "nyu"),
    NYO("ニョ", "nyo"),

    HYA("ヒャ", "hya"),
    HYU("ヒュ", "hyu"),
    HYO("ヒョ", "hyo"),

    MYA("ミャ", "mya"),
    MYU("ミュ", "myu"),
    MYO("ミョ", "myo"),

    RYA("リャ", "rya"),
    RYU("リュ", "ryu"),
    RYO("リョ", "ryo"),

    GYA("ギャ", "gya"),
    GYU("ギュ", "gyu"),
    GYO("ギョ", "gyo"),

    JA1("ジャ", "ja"),
    JU1("ジュ", "ju"),
    JO1("ジョ", "jo"),

    JA2("ヂャ", "ja"),
    JU2("ヂュ", "ju"),
    JO2("ヂョ", "jo"),

    BYA("ビャ", "bya"),
    BYU("ビュ", "byu"),
    BYO("ビョ", "byo"),

    PYA("ピャ", "pya"),
    PYU("ピュ", "pyu"),
    PYO("ピョ", "pyo"),

    LEHNWORT_F1("フォ", "fo"),
    LEHNWORT_F2("フェ", "fe"),
    LEHNWORT_F3("フィ", "fi"),
    LEHNWORT_F4("ファ", "fa"),

    LEHNWORT_W1("ヴォ", "wo"),
    LEHNWORT_W2("ヴェ", "we"),
    LEHNWORT_W3("ヴ", "w"),
    LEHNWORT_W4("ヴィ", "wi"),
    LEHNWORT_W5("ヴァ", "wa"),

    LEHNWORT_TI("ティ", "ti"),
    LEHNWORT_DI("ディ", "di"),

    LEHNWORT_TU("テュ", "tu"),
    LEHNWORT_DU("デュ", "du"),


    SMALL_TSU("ッ", "kleines tsu"),
    ;

    /** Das Zeichen in UTF-8 Darstellung. */
    private final String character;

    /** Die Hepburn-Darstellung in lateinischen Buchstaben. */
    private final String hepburn;

    /**
     * Konstruktor.
     *
     * @param character
     *            Das Zeichen in UTF-8 Darstellung.
     * @param hepburn
     *            Die Hepburn-Darstellung in lateinischen Buchstaben.
     */
    private Katakana(String character, String hepburn) {
        this.character = character;
        this.hepburn = hepburn;
    }

    /** Getter für das Zeichen in UTF-8 Darstellung. */
    public String getCharacter() {
        return character;
    }

    /** Getter für die Hepburn-Darstellung in lateinischen Buchstaben. */
    public String getHepburn() {
        return hepburn;
    }

    /** Gibt das Katakana-Zeichen als Kana-Objekt zurück. */
    public Kana toKana() {
        Kana kana = new Kana(character, hepburn);
        return kana;
    }

    /** Gibt eine Liste mit den Zeichen der Schrift Katakana (in UTF-Zeichen) zurück. */
    public static List<String> getAllKatakanaAsStringList() {
        List<String> katakanaList = new ArrayList<>();

        for (Katakana katakana : Katakana.values()) {
            katakanaList.add(katakana.getCharacter());
        }

        return katakanaList;
    }

    /** Gibt an, ob in dem Text ein Katakana-Zeichen vorkommt. */
    public static boolean containsKatakana(String text) {
        return Text.contains(text, getAllKatakanaAsStringList());
    }

    /** Gibt an, ob der Text ausschließlich aus Katakana-Zeichen besteht. */
    public static boolean containsOnlyKatakana(String text) {
        if (text.isEmpty()) {
            return false;
        }
        else {
            List<String> katakana = getAllKatakanaAsStringList();
            for (String textCharacter : Text.textToCharactersList(text)) {
                if (!katakana.contains(textCharacter)) {
                    return false;
                }
            }

            return true;
        }
    }

    /**
     * Gibt den Index des ersten Katakana-Zeichen im Text zurück. Gibt es kein solches, wird -1
     * zurückgegeben.
     */
    public static int firstIndexOfKatakana(String text) {
        List<String> katakana = getAllKatakanaAsStringList();
        List<String> textCharacters = Text.textToCharactersList(text);

        for (int index = 0; index < textCharacters.size(); ++index) {
            String textCharacter = textCharacters.get(index);
            if (katakana.contains(textCharacter )) {
                return index;
            }
        }

        return -1;
    }

    /**
     * Gibt den Index des letzten Katakana-Zeichen im Text zurück. Gibt es kein solches, wird -1
     * zurückgegeben.
     */
    public static int lastIndexOfKatakana(String text) {
        List<String> katakana = getAllKatakanaAsStringList();
        List<String> textCharacters = Text.textToCharactersList(text);

        for (int index = textCharacters.size() - 1; index >= 0; --index) {
            String textCharacter = textCharacters.get(index);
            if (katakana.contains(textCharacter )) {
                return index;
            }
        }

        return -1;
    }

}
