package de.duehl.vocabulary.japanese.logic;

import java.util.ArrayList;
import java.util.List;

import javax.swing.SwingUtilities;

import de.duehl.basics.io.FileHelper;
import de.duehl.basics.logic.ErrorHandler;
import de.duehl.basics.system.SystemTools;
import de.duehl.swing.logic.Quitter;
import de.duehl.swing.ui.GuiTools;
import de.duehl.swing.version.NewsHelper;
import de.duehl.vocabulary.japanese.VocabularyTrainerVersion;
import de.duehl.vocabulary.japanese.data.Vocable;
import de.duehl.vocabulary.japanese.data.Vocabulary;
import de.duehl.vocabulary.japanese.data.VocabularySortOrder;
import de.duehl.vocabulary.japanese.logic.internal.InternalDataRequester;
import de.duehl.vocabulary.japanese.logic.ownlists.OwnLists;
import de.duehl.vocabulary.japanese.logic.persistence.Options;
import de.duehl.vocabulary.japanese.logic.persistence.SessionManager;
import de.duehl.vocabulary.japanese.logic.translation.GermanToJapaneseTranslation;
import de.duehl.vocabulary.japanese.logic.translation.JapaneseToGermanTranslation;
import de.duehl.vocabulary.japanese.logic.translation.TranslationDirection;
import de.duehl.vocabulary.japanese.logic.wrongtested.WrongTestedVocables;
import de.duehl.vocabulary.japanese.startup.logic.StartupLoader;
import de.duehl.vocabulary.japanese.ui.VocabularyTrainerGui;
import de.duehl.vocabulary.japanese.website.update.VocabularyTrainerUpdater;

/**
 * Diese Klasse stellt die Logik des Vokabel-Trainers dar.
 *
 * @version 1.01     2024-08-21
 * @author Christian Dühl
 */

public class VocabularyTrainerLogic implements Quitter {

    /** Der Sessionmanager für Daten, die zwischen Sessions erhalten bleiben. */
    private final SessionManager sessionManager;

    /** Die Programmoptionen. */
    private final Options options;

    /** Die grafische Oberfläche. */
    private final VocabularyTrainerGui gui;

    /** Die Liste mit den bekannten Vokabularien. */
    private List<Vocabulary> vocabularies;

    /**
     * Die Nachricht mit der Anzahl Vokabeln und Vokabularien vom StartupLoader, die nach dem Laden
     * in der Statuszeile der Oberfläche angezeigt werden soll.
     */
    private String loadUpMessage;

    /** Die Verwaltung der eigenen Vokabellisten. */
    private OwnLists ownLists;

    /**
     * Das Objekt das sich um die Übersetzung von Japanisch in Deutsch kümmert (für Abfragen die
     * nur Kana, aber nicht Kanji anzeigen).
     */
    private JapaneseToGermanTranslation japaneseToGermanTranslation;

    /** Das Objekt das sich um die Übersetzung von Deutsch in Japanisch kümmert. */
    private GermanToJapaneseTranslation germanToJapaneseTranslation;

    /** Das Objekt das zu Vokabeln die internen Daten abfragt. */
    private InternalDataRequester internalDataRequester;

    /** Der Text, der beim Starten des Vokabeltrainers angezeigt wurde. */
    private String startUpLog;

    /**
     * Gibt an, dass die Prüfungen auf neue Versionen beim Start des Programms stattfinden.
     * Anderenfalls hat der Benutzer sie über das Update-Menü angestoßen.
     */
    private boolean checkingNewVersionsOnStartUp;

    /** Gibt an,. ob beim Startup ein Fehler auftrat. */
    private boolean startUpErrorOccured;

    /**
     * Die Verwaltung der beiden automatisch gepflegten Listen mit falsch abgefragten Vokabeln aus
     * Gruppen und anderen Vokabularien.
     */
    private WrongTestedVocables wrongTestedVocables;

    /** Konstruktor. */
    public VocabularyTrainerLogic() {
        sessionManager = new SessionManager();
        options = loadOptions();

        gui = new VocabularyTrainerGui(this);
        checkingNewVersionsOnStartUp = false;
    }

    private Options loadOptions() {
        try {
            return sessionManager.load();
        }
        catch (Exception exception) {
            if (gui == null) {
                System.out.println("Fehler beim Laden der Optionen: " + exception.getMessage());
                return new Options();
            }
            else {
                ErrorHandler error = gui.createErrorHandler();
                error.error(exception.getMessage(), exception);
                return new Options();
            }
        }
    }

    /** Speichert die Optionen. */
    public void saveOptions() {
        try {
            tryToSaveOption();
        }
        catch (Exception exception) {
            if (gui != null) {
                ErrorHandler error = gui.createErrorHandler();
                error.error(exception.getMessage(), exception);
            }
            else {
                throw new RuntimeException("Fehler beim Speichern der Optionen!", exception);
            }
        }
    }

    private void tryToSaveOption() {
        storeToOptions();

        /* Falls beim Aufbau des Menüs was schief geht, noch nicht definiert: */
        if (sessionManager != null) {
            sessionManager.save();
        }
    }

    private void storeToOptions() {
        /* Falls beim Aufbau des Menüs was schief geht, noch nicht definiert: */
        if (null != options) {
            if (gui != null) {
                options.setStringSelectionFontSize(gui.getStringSelectionFontSize());
                options.setStringSelectionLabelFontSize(gui.getStringSelectionLabelFontSize());
                gui.storeShownTabIndices();
            }
        }
    }

    /** Getter für die Programmoptionen. */
    public Options getOptions() {
        return options;
    }

    /** Getter für den Index des zuletzt betrachteten Reiters in der Oberfläche. */
    public int getLastShownMainTabIndex() {
        if (null != options) {
            return options.getLastShownMainTabIndex();
        }
        else {
            return new Options().getLastShownMainTabIndex();
        }
    }

    /**
     * Getter für den Index des zuletzt betrachteten Kategorie-Reiters (Vokabularien / eigene
     * Listen) in der Oberfläche.
     */
    public int getLastShownVocabularyCategoryTabIndex() {
        if (null != options) {
            return options.getLastShownVocabularyCategoryTabIndex();
        }
        else {
            return new Options().getLastShownVocabularyCategoryTabIndex();
        }
    }

    /**
     * Getter für den Index des zuletzt betrachteten Kategorie-Reiters (Vokabularien / eigene
     * Listen) in der Oberfläche.
     */
    public int getLastShownOwnListCategoryTabIndex() {
        if (null != options) {
            return options.getLastShownOwnListCategoryTabIndex();
        }
        else {
            return new Options().getLastShownOwnListCategoryTabIndex();
        }
    }

    /**
     * Getter für den Index des zuletzt betrachteten Unterkategorie-Reiters (Vokabularien / eigene
     * Listen) in der Oberfläche.
     */
    public int getLastShownVocabularySubCategoryTabIndex() {
        if (null != options) {
            return options.getLastShownVocabularySubCategoryTabIndex();
        }
        else {
            return new Options().getLastShownVocabularySubCategoryTabIndex();
        }
    }

    /**
     * Getter für den Index des zuletzt betrachteten Unterkategorie-Reiters (Vokabularien / eigene
     * Listen) in der Oberfläche.
     */
    public int getLastShownOwnListSubCategoryTabIndex() {
        if (null != options) {
            return options.getLastShownOwnListSubCategoryTabIndex();
        }
        else {
            return new Options().getLastShownOwnListSubCategoryTabIndex();
        }
    }

    /** Getter für die Anzahl der Spalten mit den Vokabularien. */
    public int getNumberOfVocabularyBarColumns() {
        if (null != options) {
            return options.getNumberOfVocabularyBarColumns();
        }
        else {
            return new Options().getNumberOfVocabularyBarColumns();
        }
    }

    /**
     * Getter für die Richtung, in die übersetzt wird (Japanisch - Deutsch oder Deutsch -
     * Japanisch).
     */
    public TranslationDirection getTranslationDirection() {
        if (null != options) {
            return options.getTranslationDirection();
        }
        else {
            return new Options().getTranslationDirection();
        }
    }

    /**
     * Gibt an, wieviele Tage 'eine Weile' ist, um Vokabeln anzuzeigen oder abzufragen, die 'lange
     * nicht mehr abgefragt' wurden.
     */
    public int getNumberOfDaysForAWhile() {
        if (null != options) {
            return options.getNumberOfDaysForAWhile();
        }
        else {
            return new Options().getNumberOfDaysForAWhile();
        }
    }

    /**
     * Gibt an wieviele Tage seit des ersten Auftretens eine Vokabel noch als 'neu' gilt, um die
     * 'neuesten' Vokabeln abzufragen oder anzuzeigen.
     */
    public int getNumberOfDaysAVocableIsNew() {
        if (null != options) {
            return options.getNumberOfDaysAVocableIsNew();
        }
        else {
            return new Options().getNumberOfDaysAVocableIsNew();
        }
    }

    /** Führt den Vokabeltrainer aus. */
    public void run() {
        if (isInitialDirectoryChoosingNecessary()) {
            initialDirectoryChoosing();
        }

        startUp();

        if (!startUpErrorOccured) {
            perhapsShowNews();

            actualizeStringSelectionSize();
            actualizeStringSelectionLabelSize();

            initGuiWithVocabulariesAndOwnLists();
            showLoadUpMessageInStatusBar();
            gui.setVisible(true);

            checksForNewVersionsAtStart();
        }
    }

    private boolean isInitialDirectoryChoosingNecessary() {
        String vocabulariesPath = options.getVocabulariesPath();
        return vocabulariesPath.isBlank()
                || !FileHelper.isDirectory(vocabulariesPath);
    }

    private void initialDirectoryChoosing() {
        String homeDir = SystemTools.getHomeDirectory();
        String vocabulariesPath = GuiTools.openDirectory(homeDir,
                "Bitte das Verzeichnis mit den Vokabeln wählen",
                gui.getWindowAsComponent());

        if (FileHelper.isDirectory(vocabulariesPath)) {
            options.setVocabulariesPath(vocabulariesPath);
        }
        else {
            System.exit(1);
        }
    }

    private void startUp() {
        startUpErrorOccured = false;
        reallyStartUp();
    }

    private void reallyStartUp() {
        try {
            tryStartUp();
        }
        catch (Exception exception) {
            reallyQuit();
            gui.createErrorHandler().error("Es trat ein Fehler beim Startup auf", exception);
            startUpErrorOccured = true;
        }
    }

    private void tryStartUp() {
        StartupLoader loader = new StartupLoader(gui, options);
        loader.load();
        vocabularies = loader.getVocabularies();
        internalDataRequester = loader.getInternalDataRequester();
        loadUpMessage = loader.getLoadUpMessage();
        japaneseToGermanTranslation = loader.getJapaneseToGermanTranslation();
        germanToJapaneseTranslation = loader.getGermanToJapaneseTranslation();
        ownLists = loader.getOwnLists();
        wrongTestedVocables = loader.getWrongTestedVocables();
        startUpLog = loader.getStartUpLog();
    }

    private void showLoadUpMessageInStatusBar() {
        gui.setMessageLater(loadUpMessage);
    }

    private void perhapsShowNews() {
        NewsHelper newsHelper = new NewsHelper(new VocabularyTrainerVersion().getVersion(),
                options.getLastUsedVersion(), gui.getChanges(), gui.getProgramImage(),
                gui.getLocation());
        newsHelper.perhapsShowNews();
    }

    private void actualizeStringSelectionSize() {
        int stringSelectionFontSize = options.getStringSelectionFontSize();
        if (stringSelectionFontSize > -1) {
            gui.setStringSelectionFontSize(stringSelectionFontSize);
        }
    }

    private void actualizeStringSelectionLabelSize() {
        int stringSelectionLabelFontSize = options.getStringSelectionLabelFontSize();
        if (stringSelectionLabelFontSize > -1) {
            gui.setStringSelectionLabelFontSize(stringSelectionLabelFontSize);
        }
    }

    private void initGuiWithVocabulariesAndOwnLists() {
        gui.initGuiWithVocabulariesAndOwnLists(vocabularies, ownLists, wrongTestedVocables);
    }

    /** Getter für das Objekt, das zu Vokabeln die internen Daten abfragt. */
    public InternalDataRequester getInternalDataRequester() {
        return internalDataRequester;
    }

    /** Beendet das Programm (von der Gui aus). */
    @Override
    public void quit() {
        if (GuiTools.askUserToQuit(gui.getWindowAsComponent())) {
            reallyQuit();
        }
    }

    /** Beendet das Programm ohne Nachfrage beim Benutzer. */
    public void reallyQuit() {
        saveOptions();
        if (null != gui) { // Bei Fehlern im Menü ist die Gui noch nicht definiert!
            gui.saveManualVocabularyOrder();
            gui.quit();
        }
    }

    /** Stellt eine Liste mit den Vokabeln aus allen Vokabularen zusammen. */
    public List<Vocable> collectVocablesOfAllVocabularies() {
        List<Vocable> allVocables = new ArrayList<>();

        for (Vocabulary vocabulary : vocabularies) {
            allVocables.addAll(vocabulary.getVocables());
        }

        return allVocables;
    }

    /**
     * Setzt in den Optionen die Art die Vokabularien in der Oberfläche zu sortieren und speichert
     * sie hinterher.
     */
    public void setVocabularySortOrder(VocabularySortOrder sortOrder) {
        options.setVocabularySortOrder(sortOrder);
        saveOptions();
    }

    /** Getter für die Liste mit den bekannten Vokabularien. */
    public List<Vocabulary> getVocabularies() {
        return vocabularies;
    }

    /**
     * Speichert die abgefragten Vokabeln als Liste.
     *
     * @param vocables
     *            Die Liste der abzuspeichernden Vokabeln.
     * @param nameSuggestion
     *            Ein Vorschlag für den Namen der Liste.
     */
    public void saveAsList(List<Vocable> vocables, String nameSuggestion) {
        ownLists.saveAsList(vocables, nameSuggestion);
    }

    /**
     * Gibt an, ob wir (im Fall der lazy initialization) das Verzeichnis für die Übersetzung von
     * Deutsch zu Japanisch noch erzeugen müssen.
     */
    public boolean doWeHaveToCreateGermanJapaneseTranslation() {
        return germanToJapaneseTranslation == null;
    }

    public void createGermanJapaneseTranslation(Runnable runAfter) {
        gui.startLongTimeProcess("Erzeuge Datenstrukturen für die Zuordnung von Kana und Kanji zu "
                + "Vokabeln sowie mehrdeutige Zuordnungen von Kana ohne Kanji zu Vokabeln für die "
                + "Übersetzung von Japanisch in Deutsch");
        new Thread(() -> createGermanJapaneseTranslationInThread(runAfter)).start();
    }

    private void createGermanJapaneseTranslationInThread(Runnable runAfter) {
        germanToJapaneseTranslation = new GermanToJapaneseTranslation(vocabularies);
        SwingUtilities.invokeLater(() -> afterCreationGermanJapaneseTranslationInEdt(runAfter));
    }

    private void afterCreationGermanJapaneseTranslationInEdt(Runnable runAfter) {
        gui.endLongTimeProcess();
        runAfter.run();
    }

    /** Gibt zu einem deutschen Begriff die passenden Vokabeln zurück. */
    public List<Vocable> getMatchingVocablesForGermanTerm(String germanTerm) {
        if (null == germanToJapaneseTranslation) {
            throw new RuntimeException("Da ging etwas mit der 'lazy initialization' von "
                    + "germanToJapaneseTranslation schief.");
        }
        return germanToJapaneseTranslation.getMatchingVocablesForGermanTerm(germanTerm);
    }

    /** Gibt die Liste aller Vokabeln zurück, welche die gesuchten Kana aufweisen. */
    public List<Vocable> getMatchingVocablesForKana(String kana) {
        return japaneseToGermanTranslation.getMatchingVocablesForKana(kana);
    }

    /**
     * Gibt die Liste aller Vokabeln zurück, welche die gesuchten Kana und Kanji aufweisen. Diese
     * sollte immer genau ein Element beinhalten.
     */
    public List<Vocable> getMatchingVocablesForKanaAndKanji(String kana, String kanji) {
        return japaneseToGermanTranslation.getMatchingVocablesForKanaAndKanji(kana, kanji);
    }

    private void checksForNewVersionsAtStart() {
        checkingNewVersionsOnStartUp = true;

        if (options.isCheckNewProgramVersionAtStart()) {
            checkNewProgramVersion();
        }
        if (options.isCheckNewVocabulariesVersionAtStart()) {
            checkNewVocabulariesVersion();
        }
        if (options.isCheckNewOwnListsVersionAtStart()) {
            checkNewOwnListsVersion();
        }

        checkingNewVersionsOnStartUp = false;
    }

    public void checkNewProgramVersion() {
        new Thread(() -> checkNewProgramVersionInOwnThread()).start();
    }

    private void checkNewProgramVersionInOwnThread() {
        VocabularyTrainerUpdater updater = createUpdater();
        updater.checkNewProgramVersion();
    }

    public void checkNewVocabulariesVersion() {
        new Thread(() -> checkNewVocabulariesVersionInOwnThread()).start();
    }

    private void checkNewVocabulariesVersionInOwnThread() {
        VocabularyTrainerUpdater updater = createUpdater();
        updater.checkNewVocabulariesVersion();
    }

    public void checkNewOwnListsVersion() {
        new Thread(() -> checkNewOwnListsVersionInOwnThread()).start();
    }

    private void checkNewOwnListsVersionInOwnThread() {
        VocabularyTrainerUpdater updater = createUpdater();
        updater.checkNewOwnListsVersion();
    }

    private VocabularyTrainerUpdater createUpdater() {
        VocabularyTrainerUpdater updater = new VocabularyTrainerUpdater(this, gui);
        if (checkingNewVersionsOnStartUp) {
            updater.checkingNewVersionsOnStartUp();
        }
        return updater;
    }

    /** Getter für den Text, der beim Starten des Vokabeltrainers angezeigt wurde. */
    public String getStartUpLog() {
        return startUpLog;
    }

    /** Fügt eine Vokabel zur passenden Liste hinzu. */
    public void addVocableToWrongTestedVocablesList(Vocable vocable) {
        wrongTestedVocables.addVocableToWrongTestedVocablesList(vocable);
        gui.setCorrectForegroundColorOfVocabularyBarsLater(); // Färbt den Text
        gui.showTranslationDirectionOnBarButtons(); // aktualisiert die Buttons (inaktiv / aktiv)
    }

    /** Entfernt eine Vokabel aus der passenden Liste. */
    public void removeVocableFromWrongTestedVocablesList(Vocable vocable) {
        wrongTestedVocables.removeVocableFromWrongTestedVocablesList(vocable);
        gui.setCorrectForegroundColorOfVocabularyBarsLater(); // Färbt den Text
        gui.showTranslationDirectionOnBarButtons(); // aktualisiert die Buttons (inaktiv / aktiv)
    }

}
