package de.duehl.vocabulary.japanese.startup.logic;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.duehl.basics.datetime.date.ImmutualDate;
import de.duehl.basics.datetime.time.watch.StopWatch;
import de.duehl.basics.io.FileHelper;
import de.duehl.basics.text.NumberString;
import de.duehl.basics.text.Text;
import de.duehl.vocabulary.japanese.data.Vocable;
import de.duehl.vocabulary.japanese.data.Vocabulary;
import de.duehl.vocabulary.japanese.logic.internal.InternalDataRequester;
import de.duehl.vocabulary.japanese.logic.internal.RealInternalDataRequester;
import de.duehl.vocabulary.japanese.logic.internal.data.InternalAdditionalVocableData;
import de.duehl.vocabulary.japanese.logic.ownlists.KeyToVocableMapCreator;
import de.duehl.vocabulary.japanese.logic.ownlists.OwnLists;
import de.duehl.vocabulary.japanese.logic.persistence.Options;
import de.duehl.vocabulary.japanese.logic.persistence.SessionManager;
import de.duehl.vocabulary.japanese.logic.translation.GermanToJapaneseTranslation;
import de.duehl.vocabulary.japanese.logic.translation.JapaneseToGermanTranslation;
import de.duehl.vocabulary.japanese.logic.wrongtested.WrongTestedVocables;
import de.duehl.vocabulary.japanese.startup.ui.MessageAppender;
import de.duehl.vocabulary.japanese.startup.ui.SplashScreen;
import de.duehl.vocabulary.japanese.tools.VocabularyTools;
import de.duehl.vocabulary.japanese.ui.VocabularyTrainerGui;

/**
 * Diese Klasse lädt beim Starten des Vokabeltrainers die Vokabulare ein, sucht die Sound-Files und
 * stellt die Verbindung zu den benutzerabhängigen, internen Daten jeder Variablen her.
 *
 * Außerdem werden Datenstrukturen aufgebaut, die vom Vokabel-Trainer benötigt werden.
 *
 * @version 1.01     2024-09-18
 * @author Christian Dühl
 */

public class StartupLoader implements MessageAppender {

    /**
     * Die Extension für den Dateien mit den benutzerabhängigen, internen Daten zu jeder Vokabel.
     */
    private static final String INTERNAL_VARIABLE_DATA_EXTENSION = ".ivd";

    public static final String INTERNAL_DATA_DIRECTORY = FileHelper.concatPathes(
            SessionManager.VOCABLE_TRAINER_DIRECTORY, "internal_vocabulary_data");


    /** Die grafische Oberfläche. */
    private final VocabularyTrainerGui gui;

    /** Die Programmoptionen. */
    private final Options options;

    /** Die Liste mit den bekannten Vokabularien. */
    private List<Vocabulary> vocabularies;

    /** Die Liste aller Schlüssel der eingelesenen internen Daten. */
    private List<String> allKeysFromReadInternalData;

    /**
     * Die Liste der Schlüssel von eingelesenen internen Daten, welche keiner Vokabel zugeordnet
     * werden konnten.
     */
    private List<String> unusedKeysFromReadInternalData;

    /** Die Liste der Schlüssel die aus einer Vokabel erzeugt wurden. */
    private List<String> seenKeysFromVocables;

    /** Die Liste der Schlüssel die identisch aus mehr als einer Vokabel erzeugt wurden. */
    private List<String> multipleCreatedKeysFromVocables;

    /** Das Verzeichnis der internen Daten zu einer Vokabel nach ihrem Schlüssel. */
    private Map<String, InternalAdditionalVocableData> key2InternalDataMap;

    /** Das Verzeichnis der internen Daten zu einer Vokabel nach der zugehörigen Variablen. */
    private Map<Vocable, InternalAdditionalVocableData> vocable2InternalDataMap;

    /** Das Objekt das zu Vokabeln die internen Daten abfragt. */
    private InternalDataRequester internalDataRequester;

    /**
     * Die Nachricht mit der Anzahl Vokabeln und Vokabularien, die nach dem Laden in der
     * Statuszeile der Oberfläche angezeigt werden soll.
     */
    private String loadUpMessage;

    /** Zählt, wie viele interne Daten neu angelegt wurden. */
    private int newInternalDataCreationCounter;

    /** Das Objekt das sich um die Übersetzung von Deutsch in Japanisch kümmert. */
    private GermanToJapaneseTranslation germanToJapaneseTranslation;

    /**
     * Das Objekt das sich um die Übersetzung von Japanisch in Deutsch kümmert (für Abfragen die
     * nur Kana, aber nicht Kanji anzeigen).
     */
    private JapaneseToGermanTranslation japaneseToGermanTranslation;

    /** Die Verwaltung der eigenen Vokabellisten. */
    private OwnLists ownLists;

    /**
     * Die Verwaltung der beiden automatisch gepflegten Listen mit falsch abgefragten Vokabeln aus
     * Gruppen und anderen Vokabularien.
     */
    private WrongTestedVocables wrongTestedVocables;

    /** Die grafische Oberfläche beim Start in der die Meldungen angezeigt werden. */
    private final SplashScreen splashScreen;

    /** Misst die Laufzeit. */
    private StopWatch watch;

    /**
     * Konstruktor.
     *
     * @param gui
     *            Die grafische Oberfläche.
     * @param options
     *            Die Programmoptionen.
     */
    public StartupLoader(VocabularyTrainerGui gui, Options options) {
        this.gui = gui;
        this.options = options;
        splashScreen = new SplashScreen(gui.getProgramImage());
    }

    /** Führt das Laden, Suchen und Verknüpfen durch. */
    public void load() {
        init();
        showSplashScreen();
        startup();
        closeSplashScreen();
    }

    private void init() {
        watch = new StopWatch();
    }

    private void showSplashScreen() {
        splashScreen.setVisible(true);
    }

    private void startup() {
        readVocabularies();
        readInternalVocableData();
        joinVocablesWithInternalData();
        createInternalDataRequester();
        storeDateOfOldestVocableInEachVocabulary();
        createJapaneseToGermanTranslation();
        createGermanToJapaneseTranslation();
        initOwnListsAndReadWrongTestedVocableLists();
    }

    private void readVocabularies() {
        appendMessage("Lese Vokabularien ...");
        VocabularyAndSoundFileFinder finder = new VocabularyAndSoundFileFinder(options,
                (MessageAppender) this);
        finder.find();
        vocabularies = finder.getVocabularies();
        loadUpMessage = finder.getLoadUpMessage();
    }

    /** Fügt eine Nachricht hinzu. */
    @Override
    public void appendMessage(String message) {
        String realMessage;
        if (options.isShowTimestampsInStartup()) {
            if (message.isBlank()) {
                realMessage = message;
            }
            else {
                String timeIntro = watch.getTime() + " - ";

                if (message.startsWith(" ")) {
                    int index = Text.findIndexOfFirstNonSpace(message);
                    if (index == -1) {
                        throw new RuntimeException("Kann nicht sein, die Nachricht beginnt mit "
                                + "einem Leerzeichen, ist nicht blank, aber ich finde nicht den "
                                + "Index des ersten nicht Leerzeichens.");
                    }
                    String spaces = message.substring(0, index);
                    String rest = message.substring(index);
                    realMessage = spaces + timeIntro + rest;
                }
                else {
                    realMessage = timeIntro + message;
                }
            }
        }
        else {
            realMessage = message;
        }
        splashScreen.appendMessage(realMessage);
    }

    /** Lädt alle bereits bekannten internen Datensätze zu Vokabeln. */
    private void readInternalVocableData() {
        appendMessage("Lese interne benutzerabhängige Daten zu allen Vokabeln ...");
        FileHelper.createDirectoryIfNotExists(INTERNAL_DATA_DIRECTORY);

        List<String> internalFiles = FileHelper.findFilesInMainDirectoryNio2(
                INTERNAL_DATA_DIRECTORY, INTERNAL_VARIABLE_DATA_EXTENSION);
        appendMessage(NumberString.taupu(internalFiles.size()) + " interne Datensätze gefunden.");

        List<InternalAdditionalVocableData> internalAdditionalVocableDatas = new ArrayList<>();
        for (String filename : internalFiles) {
            InternalAdditionalVocableData data = InternalAdditionalVocableData.load(filename);
            internalAdditionalVocableDatas.add(data);
        }
        appendMessage(NumberString.taupu(internalAdditionalVocableDatas.size())
                + " interne Datensätze eingelesen.");

        key2InternalDataMap = new HashMap<>();
        allKeysFromReadInternalData = new ArrayList<>();
        for (InternalAdditionalVocableData data : internalAdditionalVocableDatas) {
            String key = data.getKey();
            if (key2InternalDataMap.containsKey(key) || allKeysFromReadInternalData.contains(key)) {
                throw new RuntimeException("Der Schlüssel '" + key + "' kommt mehrfach vor!");
            }

            key2InternalDataMap.put(key, data);
            allKeysFromReadInternalData.add(key);
        }
        appendMessage("Verzeichnis mit " + NumberString.taupu(key2InternalDataMap.size())
                + " Verweisen von Schlüssel auf interne Datensätze aufgebaut.");
    }

    /**
     * Verbindet die Vokabeln mit den internen Daten.
     *
     * Dabei wird darauf auf die folgenden Umstände geachtet:
     *
     *     1) Ob dabei Schlüssel aus den eingelesenen internen Daten übrig bleiben, was darauf
     *        hindeutet, dass man an den Vokabeln etwas geändert hat und diese internen Daten
     *        entfernt werden sollten, ggf., nachdem man Daten daraus in die neu erzeugten
     *        internen Daten übernommen hat.
     *        Die erfolgt über die Liste
     *            unusedKeysFromReadInternalData.
     *     2) Es wird geprüft, dass keine zwei verschiedenen Vokabeln zum gleichen Schlüssel
     *        führen.
     *        Durch die Prüfung in
     *            VocabularyAndSoundFileFinder#
     *                checkIfVocablesAreUniqueWithKeyFromKanaAndFirstTranslations()
     *        wissen wir zwar, dass die Kombination von Kana und der ersten Übersetzung eindeutig
     *        ist. Auch der von
     *            VocabularyTools#createVocableKey()
     *        erzeugte Schlüssel basiert auf diesen beiden Daten, allerdings werden am Schlüssel
     *        einige Ersetzungen vorgenommen, so dass man nicht mehr sicher sein kann, dass der
     *        Schlüssel wirklich eindeutig ist.
     *        Also wird geprüft, dass es nicht vorkommt, dass die Schlüssel zweier
     *        unterschiedlicher Vokabeln dennoch gleich sind.
     *        Die erfolgt über die Listen
     *            seenKeysFromVocables und
     *            multipleCreatedKeysFromVocables.
     */
    private void joinVocablesWithInternalData() {
        appendMessage("Verbinde die Vokabeln mit den internen benutzerabhängigen Daten ...");
        vocable2InternalDataMap = new HashMap<>();
        newInternalDataCreationCounter = 0;

        unusedKeysFromReadInternalData = new ArrayList<>();
        unusedKeysFromReadInternalData.addAll(allKeysFromReadInternalData);

        seenKeysFromVocables = new ArrayList<>();
        multipleCreatedKeysFromVocables = new ArrayList<>();

        for (Vocabulary vocabulary : vocabularies) {
            for (Vocable vocable : vocabulary.getVocables()) {
                joinVocableInternalData(vocable);
            }
        }

        appendMessage("Verzeichnis mit " + NumberString.taupu(vocable2InternalDataMap.size())
                + " Verweisen von Vokabeln auf interne Datensätze aufgebaut.");
        appendMessage(NumberString.taupu(newInternalDataCreationCounter)
                + " neue interne Daten angelegt.");

        if (unusedKeysFromReadInternalData.isEmpty()) {
            appendMessage("Alle interne Daten wurden einer Vokabel zugeordnet.");
        }
        else {
            appendMessage("Es gibt " + NumberString.taupu(unusedKeysFromReadInternalData.size())
                    + " interne Daten, welcher keiner Vokabel zugeordnet wurden:");
            for (String unusedKey : unusedKeysFromReadInternalData) {
                appendMessage("    " + unusedKey);
            }
        }

        if (multipleCreatedKeysFromVocables.isEmpty()) {
            appendMessage("Alle Vokabeln erzeugen unterschiedliche Schlüssel.");
        }
        else {
            appendMessage("Es gibt " + NumberString.taupu(multipleCreatedKeysFromVocables.size())
                    + " Schlüssel, welche zu mehr als einer Vokabel gehören");
            for (String multipleCreatedKey : multipleCreatedKeysFromVocables) {
                appendMessage("    " + multipleCreatedKey);
            }

            // Kann ja nur bei mir auftreten, daher:
            System.err.println("Daher wird der Start abgebrochen.");
            System.exit(1);
        }
    }

    /** Verbindet die übergebene Vokabel über den Schlüssel mit ihren internen Daten. */
    private void joinVocableInternalData(Vocable vocable) {
        String key = VocabularyTools.createVocableKey(vocable);
        unusedKeysFromReadInternalData.remove(key); // dieser interne Schlüssel wird verwendet!

        if (seenKeysFromVocables.contains(key)) {
            if (!multipleCreatedKeysFromVocables.contains(key)) {
                multipleCreatedKeysFromVocables.add(key);
            }
        }
        else {
            seenKeysFromVocables.add(key);
        }

        InternalAdditionalVocableData data;
        if (key2InternalDataMap.containsKey(key)) {
            data = key2InternalDataMap.get(key);
        }
        else {
            data = new InternalAdditionalVocableData();
            data.setKey(key);
            String internalDataFilename = FileHelper.concatPathes(INTERNAL_DATA_DIRECTORY,
                    key + INTERNAL_VARIABLE_DATA_EXTENSION);
            data.setFilename(internalDataFilename);
            data.setFirstSeenDate(new ImmutualDate()); // heute

            appendMessage("neu: " + internalDataFilename);
            ++newInternalDataCreationCounter;
            data.save();
        }

        vocable2InternalDataMap.put(vocable, data);
    }

    private void createInternalDataRequester() {
        internalDataRequester = new RealInternalDataRequester(vocable2InternalDataMap);
    }

    private void storeDateOfOldestVocableInEachVocabulary() {
        appendMessage("Trage Datum der ältesten Vokabel in allen Vokabularien ein ...");
        for (Vocabulary vocabulary : vocabularies) {
            storeDateOfOldestVocableInVocabulary(vocabulary);
        }
    }

    private void storeDateOfOldestVocableInVocabulary(Vocabulary vocabulary) {
        ImmutualDate oldestDate = new ImmutualDate();

        for (Vocable vocable : vocabulary.getVocables()) {
            InternalAdditionalVocableData data = vocable2InternalDataMap.get(vocable);
            ImmutualDate vocableFirstSeenDate = data.getFirstSeenDate();
            if (!vocableFirstSeenDate.equals(InternalAdditionalVocableData.NOT_SEEN_DATE)
                    && vocableFirstSeenDate.before(oldestDate)) {
                oldestDate = vocableFirstSeenDate;
            }
        }

        vocabulary.setFirstSeenDate(oldestDate);
    }

    private void createJapaneseToGermanTranslation() {
        appendMessage("Erzeuge Datenstrukturen für die Zuordnung von Kana und Kanji zu Vokabeln "
                + "sowie mehrdeutige Zuordnungen von Kana ohne Kanji zu Vokabeln für die "
                + "Übersetzung von Japanisch in Deutsch ...");
        japaneseToGermanTranslation = new JapaneseToGermanTranslation(vocabularies);
    }

    private void createGermanToJapaneseTranslation() {
        if (options.isCreateGermanJapaneseTranslationAtStartup()) {
            appendMessage("Erzeuge Datenstrukturen für mehrdeutige Übersetzungen von Deutsch in "
                    + "Japanisch ...");
            germanToJapaneseTranslation = new GermanToJapaneseTranslation(vocabularies);
        }
        else {
            appendMessage("Die Datenstrukturen für mehrdeutige Übersetzungen von Deutsch in "
                    + "Japanisch werden noch nicht erzeugt, sondern erst bei der ersten "
                    + "Verwendung.");
        }
    }

    private void initOwnListsAndReadWrongTestedVocableLists() {
        appendMessage("Erzeuge Verzeichnis Schlüssel zu Vokabel ...");
        KeyToVocableMapCreator creator = new KeyToVocableMapCreator(vocabularies,
                internalDataRequester);
        creator.create();
        Map<String, Vocable> keyToVocable = creator.getKeyToVocable();

        appendMessage("Erzeuge Verwaltung der eigene Listen ...");
        ownLists = new OwnLists(vocabularies, keyToVocable, internalDataRequester, options, gui);

        appendMessage("Erzeuge Verwaltung der automatisch gepflegten Listen der falsch übersetzten "
                + "Vokabeln ...");
        wrongTestedVocables = new WrongTestedVocables(vocabularies, keyToVocable,
                internalDataRequester);
    }

    private void closeSplashScreen() {
        splashScreen.closeDialog();
    }

    /**
     * Getter für die Nachricht mit der Anzahl Vokabeln und Vokabularien, die nach dem Laden in der
     * Statuszeile der Oberfläche angezeigt werden soll.
     */
    public String getLoadUpMessage() {
        return loadUpMessage;
    }

    /** Getter für die Liste mit den bekannten Vokabularien. */
    public List<Vocabulary> getVocabularies() {
        return vocabularies;
    }

    /** Getter für das Objekt, das zu Vokabeln die internen Daten abfragt. */
    public InternalDataRequester getInternalDataRequester() {
        return internalDataRequester;
    }

    /**
     * Getter für das Objekt das sich um die Übersetzung von Japanisch in Deutsch kümmert (für
     * Abfragen die nur Kana, aber nicht Kanji anzeigen).
     */
    public JapaneseToGermanTranslation getJapaneseToGermanTranslation() {
        return japaneseToGermanTranslation;
    }

    /** Getter für das Objekt das sich um die Übersetzung von Deutsch in Japanisch kümmert. */
    public GermanToJapaneseTranslation getGermanToJapaneseTranslation() {
        return germanToJapaneseTranslation;
    }

    /** Getter für die Verwaltung der eigenen Vokabellisten. */
    public OwnLists getOwnLists() {
        return ownLists;
    }

    /**
     * Getter für die Verwaltung der beiden automatisch gepflegten Listen mit falsch abgefragten
     * Vokabeln aus Gruppen und anderen Vokabularien.
     */
    public WrongTestedVocables getWrongTestedVocables() {
        return wrongTestedVocables;
    }

    /** Getter für den Text, der beim Starten des Vokabeltrainers angezeigt wurde. */
    public String getStartUpLog() {
        return splashScreen.getSplashText();
    }

}
