package de.duehl.vocabulary.japanese.ui.components;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;

import javax.swing.JPanel;
import javax.swing.JTextPane;

import de.duehl.basics.io.FileHelper;
import de.duehl.swing.ui.buttons.painted.GoButton;
import de.duehl.swing.ui.layout.VerticalLayout;
import de.duehl.vocabulary.japanese.data.Vocable;
import de.duehl.vocabulary.japanese.logic.color.VocableColors;
import de.duehl.vocabulary.japanese.logic.internal.data.InternalAdditionalVocableData;
import de.duehl.vocabulary.japanese.logic.persistence.Options;
import de.duehl.vocabulary.japanese.tools.VocabularyTools;
import de.duehl.vocabulary.japanese.ui.components.display.KanjiAndKanaDisplay;
import de.duehl.vocabulary.japanese.ui.components.display.RomajiAndPronunciationDisplay;
import de.duehl.vocabulary.japanese.ui.components.display.TranslationCommentAndVocabularyDescriptionDisplay;

/**
 * Diese Klasse zeigt eine Vokabel an.
 *
 * @version 1.01     2024-08-15
 * @author Christian Dühl
 */

public class VocableViewer {

    /** Die Programmoptionen. */
    private final Options options;

    /** Der Panel auf dem die Vokabel dargestellt wird. */
    private final JPanel panel;

    /** Die TextPane in der die Kanji und Kana dargestellt werden. */
    private final JTextPane kanjiAndKanaPane;

    /** Die TextPane in der die Romaji und Aussprache dargestellt werden. */
    private final JTextPane romajiAndPronunciationPane;

    /**
     * Die TextPane in der die Übersetzungen, den Kommentar und die Beschreibung des Vokabulars
     * dargestellt werden.
     */
    private final JTextPane translationCommentAndVocabularyDescriptionPane;

    private final GoButton playMp3Button;

    /** Die Vokabel, welche angezeigt wird. */
    private Vocable vocable;

    /**
     * Konstruktor.
     *
     * @param options
     *            Die Programmoptionen.
     */
    public VocableViewer(Options options) {
        this.options = options;

        panel = new JPanel();
        kanjiAndKanaPane = new JTextPane();
        romajiAndPronunciationPane = new JTextPane();
        translationCommentAndVocabularyDescriptionPane = new JTextPane();
        playMp3Button = new GoButton();

        initSelections();
        createPanel();
    }

    private void initSelections() {
        playMp3Button.addActionListener(e -> playMp3());
        playMp3Button.setForceSquare(true);
        playMp3Button.setPreferredSize(new Dimension(50, 50));
    }

    private void createPanel() {
        panel.setLayout(new VerticalLayout(0, VerticalLayout.BOTH));

        panel.add(createKanjiKanaAndMp3PlayButtonArea());
        panel.add(romajiAndPronunciationPane);
        panel.add(translationCommentAndVocabularyDescriptionPane);
    }

    private Component createKanjiKanaAndMp3PlayButtonArea() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());

        panel.add(kanjiAndKanaPane, BorderLayout.CENTER);
        panel.add(playMp3Button, BorderLayout.EAST);

        return panel;
    }


    /** Getter für den Panel auf dem die Vokabel dargestellt wird. */
    public Component getPanel() {
        return panel;
    }

    /** Zeigt eine neue Variable an. */
    public void showVocable(Vocable vocable, InternalAdditionalVocableData data) {
        this.vocable = vocable;

        VocableColors vocableColors = new VocableColors(options);
        Color backgroundColor = vocableColors.determineBackgroundColor(data,
                options.isColorVocableDependingOnLastSuccessWhenViewingVocable(),
                options.getTranslationDirection());

        KanjiAndKanaDisplay display1 = new KanjiAndKanaDisplay(vocable, kanjiAndKanaPane,
                backgroundColor, options);
        display1.display();

        playMp3Button.setBackgroundColor(backgroundColor);

        RomajiAndPronunciationDisplay display2 = new RomajiAndPronunciationDisplay(vocable,
                romajiAndPronunciationPane, backgroundColor, options);
        display2.display();

        TranslationCommentAndVocabularyDescriptionDisplay display3 =
                new TranslationCommentAndVocabularyDescriptionDisplay(vocable,
                        translationCommentAndVocabularyDescriptionPane, backgroundColor, options);
        display3.display();

        String mp3 = vocable.getMp3();
        playMp3Button.setEnabled(!mp3.isBlank() && FileHelper.isFile(mp3));

        panel.repaint();
    }

    private void playMp3() {
        String mp3 = vocable.getMp3();
        VocabularyTools.playMp3(mp3);
    }

}
