package de.duehl.vocabulary.japanese.ui.components.bars;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.FlowLayout;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import de.duehl.swing.ui.GuiTools;
import de.duehl.swing.ui.buttons.painted.DownButton;
import de.duehl.swing.ui.buttons.painted.FirstButton;
import de.duehl.swing.ui.buttons.painted.LastButton;
import de.duehl.swing.ui.buttons.painted.UpButton;
import de.duehl.swing.ui.colors.ColorTool;
import de.duehl.vocabulary.japanese.data.Vocable;
import de.duehl.vocabulary.japanese.ui.components.text.VobaleBarTextCreator;
import de.duehl.vocabulary.japanese.ui.dialog.ownlist.OwnListEditorDialog;

import static de.duehl.vocabulary.japanese.ui.components.bars.VocabularyBar.BUTTON_DIMENSION;
import static de.duehl.vocabulary.japanese.ui.components.bars.VocabularyBar.DISABLED_COLOR;
import static de.duehl.vocabulary.japanese.ui.components.bars.VocabularyBar.MOVE_BUTTON_COLOR;

/**
 * Diese Klasse stellt die grafische Oberfläche der Anzeige einer Vokabel in einer eigenen
 * Vokabelliste oder der Liste mit der Auswahl der Vokabeln dazu dar.
 *
 * Dieser wird nur in dem Dialog zum Bearbeiten einer Liste (OwnListEditorDialog) und im Dialog
 * zur erweiterten Suche (ComplexVocableSearchDialog) angezeigt.
 *
 * @version 1.01     2024-08-22
 * @author Christian Dühl
 */

public class VocableBar {

    /** Die angezeigte Vokabel. */
    private final Vocable vocable;

    /** Wird ausgeführt, wenn der Button gedrückt wird. */
    private final Runnable buttonAction;

    /** Der Dialog, in dem die Bars bewegt etc. werden. */
    private final OwnListEditorDialog gui;

    /** Der Panel auf dem die Vokabel dargestellt wird. */
    private final JPanel panel;

    /** Der Panel auf dem der Schalter und vielleicht auch die CheckBox dargestellt wird. */
    private final JPanel rightPanel;

    /** Das Label, auf dem die Vokabel dargestellt wird. */
    private final JLabel vocableLabel;

    /** CheckBox zur Selektion. */
    private final JCheckBox selectionCheckBox;

    /** Der Multifunktions-Button (Hinzufügen zur Liste oder Löschen aus Liste). */
    private final JButton multifunctionButton;

    /** Der Panel rechts mit Buttons zum Verschieben. */
    private final JPanel moveButtonsPanel;

    /** Der Schalter um die Vokabelliste nach oben zu verschieben. */
    private final UpButton upButton;

    /** Der Schalter um die Vokabelliste nach unten zu verschieben. */
    private final DownButton downButton;

    /**
     * Gibt an, ob die Schalter zum Bewegen der Bar genutzt werden (nur dann muss gui definiert
     * sein). Dies braucht man für den Dialog zum Bearbeiten der eigenen Listen, nicht aber für die
     * erweiterte Suche.
     */
    private boolean canBeMooved;

    /**
     * Konstruktor für die erweiterte Suche (ComplexVocableSearchDialog). Dort wird der Button
     * nicht verwendet.
     *
     * @param vocable
     *            Die angezeigte Vokabel.
     * @param buttonAction
     *            Wird ausgeführt, wenn der Button gedrückt wird.
     */
    public VocableBar(Vocable vocable, Runnable buttonAction) {
        this(vocable, buttonAction, null);
    }

    /**
     * Konstruktor für den Dialog zur Bearbeitung eigener Listen (OwnListEditorDialog) (rechte oder
     * linke Seite).
     *
     * @param vocable
     *            Die angezeigte Vokabel.
     * @param buttonAction
     *            Wird ausgeführt, wenn der Button gedrückt wird.
     * @param gui
     *            Der Dialog, in dem die Bars bewegt etc. werden.
     */
    public VocableBar(Vocable vocable, Runnable buttonAction, OwnListEditorDialog gui) {
        this.vocable = vocable;
        this.buttonAction = buttonAction;
        this.gui = gui;

        panel = new JPanel();
        vocableLabel = new JLabel();
        rightPanel = new JPanel();
        multifunctionButton = new JButton();
        selectionCheckBox = new JCheckBox();
        moveButtonsPanel = new JPanel();

        upButton = new UpButton(ColorTool.getColorByName(DISABLED_COLOR));
        downButton = new DownButton(ColorTool.getColorByName(DISABLED_COLOR));
    }

    /**
     * Nutzt den Button im Dialog zur Bearbeitung eigener Listen auf der linken Seite zum
     * Hinzufügen und zeigt auch die CheckBox zur Auswahl der Vokabel an.
     */
    public void useButtonAsAddVocableToList() {
        multifunctionButton.setText("hinzufügen");
        rightPanel.add(selectionCheckBox, BorderLayout.WEST);
        canBeMooved = true;
    }

    /**
     * Nutzt den Button im Dialog zur Bearbeitung eigener Listen auf der rechten Seite zum Löschen.
     * Die CheckBox wird nicht angezeigt.
     */
    public void useButtonAsDeletion() {
        multifunctionButton.setText("entfernen");
        canBeMooved = true;
    }

    /** Versteckt den Button für die Benutzung in der erweiterten Liste. */
    public void hideButton() {
        multifunctionButton.setText("Details ansehen");
        canBeMooved = false;
    }

    /** Baut die Gui auf. */
    public void createGui() {
        init();
        fillPanel();
    }

    /** Kennzeichnet die Vokabel als ausgegraut (sie ist bereits in der Liste). */
    public void disable() {
        multifunctionButton.setEnabled(false);
        selectionCheckBox.setSelected(false);
        selectionCheckBox.setEnabled(false);
        vocableLabel.setForeground(Color.LIGHT_GRAY);
    }

    /** Kennzeichnet die Vokabel als aktiv (sie ist nicht in der Liste). */
    public void enable() {
        multifunctionButton.setEnabled(true);
        selectionCheckBox.setEnabled(true);
        vocableLabel.setForeground(Color.BLACK);
    }

    private void init() {
        initVocableLabel();
        initButton();
        initRightPanel();
        if (canBeMooved) {
            initShowOrHidePanels();
            initUpAndDownButtons();
        }
    }

    private void initVocableLabel() {
        vocableLabel.setText(createVocableText());
        GuiTools.biggerFont(vocableLabel, 7);
        vocableLabel.setHorizontalAlignment(JLabel.CENTER);
    }

    private void initButton() {
        multifunctionButton.addActionListener(e -> buttonAction.run());
    }

    private void initRightPanel() {
        rightPanel.setLayout(new BorderLayout());
        rightPanel.add(multifunctionButton, BorderLayout.EAST);
    }

    private void initShowOrHidePanels() {
        moveButtonsPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 2, 0));
    }

    private void initUpAndDownButtons() {
        upButton.forceSquare();
        upButton.setPreferredSize(BUTTON_DIMENSION);
        upButton.addActionListener(e -> gui.moveBarUp(vocable));

        downButton.forceSquare();
        downButton.setPreferredSize(BUTTON_DIMENSION);
        downButton.addActionListener(e -> gui.moveBarDown(vocable));

        setUpAndDownButtonColorsAndEnabled();
    }

    private void populateMoveButtonsPanel(boolean showDeleteAndExportButtonsOnBars) {
        moveButtonsPanel.removeAll();

        if (showDeleteAndExportButtonsOnBars) {
            moveButtonsPanel.add(createMoveToTopButton());
            moveButtonsPanel.add(upButton);
            moveButtonsPanel.add(downButton);
            moveButtonsPanel.add(createMoveToBottomButton());
        }
    }

    private Component createMoveToTopButton() {
        FirstButton button = new FirstButton(ColorTool.getColorByName(MOVE_BUTTON_COLOR));
        button.setPreferredSize(BUTTON_DIMENSION);
        button.setHorizontal(false);
        button.addActionListener(e -> gui.moveBarToFirst(vocable));
        return button;
    }

    private Component createMoveToBottomButton() {
        LastButton button = new LastButton(ColorTool.getColorByName(MOVE_BUTTON_COLOR));
        button.setPreferredSize(BUTTON_DIMENSION);
        button.setHorizontal(false);
        button.addActionListener(e -> gui.moveBarToLast(vocable));
        return button;
    }

    public void setUpAndDownButtonColorsAndEnabled() {
        boolean canMoveUp = gui.canBarMoveUp(vocable);
        Color upButtonColor = canMoveUp
                ? ColorTool.getColorByName(MOVE_BUTTON_COLOR)
                : ColorTool.getColorByName(DISABLED_COLOR);
        upButton.setForegroundColor(upButtonColor);
        upButton.setEnabled(canMoveUp);
        upButton.repaint();

        boolean canMoveDown = gui.canBarMoveDown(vocable);
        Color downButtonColor = canMoveDown
                ? ColorTool.getColorByName(MOVE_BUTTON_COLOR)
                : ColorTool.getColorByName(DISABLED_COLOR);
        downButton.setForegroundColor(downButtonColor);
        downButton.setEnabled(canMoveDown);
        downButton.repaint();
    }

    private void fillPanel() {
        panel.setLayout(new BorderLayout());
        GuiTools.createTitle(panel);

        panel.add(vocableLabel, BorderLayout.CENTER);
        panel.add(createButtonPart(), BorderLayout.EAST);
    }

    private Component createButtonPart() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());

        panel.add(rightPanel, BorderLayout.WEST);
        panel.add(moveButtonsPanel, BorderLayout.EAST);

        return panel;
    }

    private String createVocableText() {
        VobaleBarTextCreator creator = new VobaleBarTextCreator(vocable);
        creator.create();
        return creator.getText();
    }

    /** Getter für den Panel auf dem die Vokabel dargestellt wird. */
    public Component getPanel() {
        return panel;
    }

    /** Setzt die Vokabel aus aus- oder nicht ausgewählt. */
    public void setSelected(boolean selected) {
        selectionCheckBox.setSelected(selected);
    }

    /** Gibt an, ob die Vokabel ausgewählt ist. */
    public boolean isSelected() {
        return selectionCheckBox.isSelected();
    }

    public void showMoveButtonsOnBars(boolean showMoveButtonsOnBars) {
        populateMoveButtonsPanel(showMoveButtonsOnBars);
    }

}
