package de.duehl.vocabulary.japanese.ui.components.bars;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;

import de.duehl.swing.logic.LongTimeProcessInformer;
import de.duehl.swing.ui.GuiTools;
import de.duehl.swing.ui.buttons.painted.DownButton;
import de.duehl.swing.ui.buttons.painted.FirstButton;
import de.duehl.swing.ui.buttons.painted.LastButton;
import de.duehl.swing.ui.buttons.painted.UpButton;
import de.duehl.swing.ui.colors.ColorTool;
import de.duehl.swing.ui.colors.NamedColorListFabric;
import de.duehl.vocabulary.japanese.data.OwnList;
import de.duehl.vocabulary.japanese.data.Vocabulary;
import de.duehl.vocabulary.japanese.logic.VocabularyTrainerLogic;
import de.duehl.vocabulary.japanese.logic.internal.InternalDataRequester;
import de.duehl.vocabulary.japanese.logic.persistence.Options;
import de.duehl.vocabulary.japanese.logic.success.VocabularyTestSuccesssCalculator;
import de.duehl.vocabulary.japanese.logic.test.VocableListTesterLogic;
import de.duehl.vocabulary.japanese.logic.translation.TranslationDirection;
import de.duehl.vocabulary.japanese.logic.view.CompleteVocabularyViewerLogic;
import de.duehl.vocabulary.japanese.tools.VocabularyTools;
import de.duehl.vocabulary.japanese.ui.VocabularyTrainerGui;
import de.duehl.vocabulary.japanese.ui.components.bars.data.VocabularyBarShownButtonsType;
import de.duehl.vocabulary.japanese.ui.dialog.VocabularyListerDialog;
import de.duehl.vocabulary.japanese.ui.dialog.VocabularySheetDialog;
import de.duehl.vocabulary.japanese.ui.resources.IconDefinitions;
import de.duehl.vocabulary.japanese.ui.tabs.MovingGui;
import de.duehl.vocabulary.japanese.ui.tools.VocabularyTrainerUiTools;

/**
 * Diese Klasse stellt die grafische Oberfläche der Anzeige eines Vokabulars in der Übersicht dar.
 *
 * @version 1.01     2024-08-21
 * @author Christian Dühl
 */

public class VocabularyBar {

    public static final String MOVE_BUTTON_COLOR = NamedColorListFabric.DARKGREEN;
    public static final String DISABLED_COLOR = NamedColorListFabric.DARKGRAY;
    public static final Dimension BUTTON_DIMENSION = new Dimension(35, 35);

    private static final String TEST_VIEW_LIST_SHEET_BUTTONS_NAME = "TEST-VIEW-LIST-SHEET-BUTTONS";
    private static final String MOVE_UP_AND_DOWN_BUTTONS_NAME = "MOVE-UP-AND-DOWN-BUTTONS";
    private static final String OWN_LISTS_EXTRA_BUTTONS_NAME = "OWN-LISTS-EXTRA-BUTTONS";


    /** Das Vokabular zu welchem die Bar angezeigt wird. */
    private final Vocabulary vocabulary;

    /** Die Logik des Vokabel-Trainers. */
    private final VocabularyTrainerLogic logic;

    /** Die Programmoptionen. */
    private final Options options;

    /** Das Objekt das zu einer Vokabel die internen, benutzerabhängigen Daten abrufen kann. */
    private final InternalDataRequester requester;

    /** Die grafische Oberfläche. */
    private final VocabularyTrainerGui gui;

    /** Die grafische Oberfläche die Bars bewegt und angeben kann, ob die Bewegung zulässig ist. */
    private MovingGui movingGui;

    /** Der Panel auf dem das Vokabular dargestellt wird. */
    private final JPanel panel;

    /** Der Panel mit den Buttons zum Ansehen und Anfragen des Vokabulars. */
    private final JPanel testViewListSheetButtonsPanel;

    /**
     * Der Panel der entweder den Panel mit den Buttons zum Ansehen und Anfragen des Vokabulars
     * oder aber den Panel mit Buttons zum Verschieben nach oben und unten anzeigen kann.
     */
    private final JPanel switchingContentButtonPanel;

    /** Das Layout für den Panel mit den wechselnden Inhalten (switchingContentButtonPanel). */
    private final CardLayout cardLayout;

    /** Der Panel mit Buttons zum Verschieben nach oben und unten. */
    private final JPanel moveUpAndDownButtonsPanel;

    /** Der Panel mit Buttons zum Exportieren, Bearbeiten und Löschen von eigenen Listen. */
    private final JPanel ownListsExtraButtonsPanel;

    /** Der Schalter um das Vokabular an die oberste Stelle zu verschieben. */
    private final FirstButton firstButton;

    /** Der Schalter um das Vokabular nach oben zu verschieben. */
    private final UpButton upButton;

    /** Der Schalter um das Vokabular nach unten zu verschieben. */
    private final DownButton downButton;

    /** Der Schalter um das Vokabular an die unterste Stelle zu verschieben. */
    private final LastButton lastButton;

    /** Hiermit kann man die Vokabeln dieses Vokabulars auswählen. */
    private final JCheckBox selectionCheckBox;

    /** Das Label mit der Beschreibung des Vokabulars. */
    private final JLabel vocabularyLabel;

    /** Die ggf. um den Anfang gekürzte Beschreibung des Vokabulars. */
    private String perhapsShortenedVocabularyDescription;

    /** Der Panel mit dem Titel und vielleicht der Prozentzahl des Erfolges. */
    private final JPanel vocabularyAndPercentPanel;

    /** Das Label mit dem Erfolg bei den letzten zehn Abfragen der Vokabeln. */
    private final JLabel successPercentLabel;

    /** Gibt an, welche Buttons gerade angezeigt werden. */
    private VocabularyBarShownButtonsType shownButtonsType;

    /**
     * Konstruktor.
     *
     * @param vocabulary
     *            Das Vokabular zu welchem die Bar angezeigt wird.
     * @param logic
     *            Die Logik des Vokabel-Trainers.
     * @param gui
     *            Die grafische Oberfläche.
     * @param movingGui
     *            Die grafische Oberfläche die Bars bewegt und angeben kann, ob die Bewegung
     *            zulässig ist.
     */
    public VocabularyBar(Vocabulary vocabulary, VocabularyTrainerLogic logic,
            VocabularyTrainerGui gui, MovingGui movingGui) {
        this.vocabulary = vocabulary;
        this.logic = logic;
        this.options = logic.getOptions();
        this.requester = logic.getInternalDataRequester();
        this.gui = gui;
        this.movingGui = movingGui;

        panel = new JPanel();
        moveUpAndDownButtonsPanel = new JPanel();
        selectionCheckBox = new JCheckBox();

        upButton = new UpButton(ColorTool.getColorByName(DISABLED_COLOR));
        downButton = new DownButton(ColorTool.getColorByName(DISABLED_COLOR));
        firstButton = new FirstButton(ColorTool.getColorByName(MOVE_BUTTON_COLOR));
        lastButton = new LastButton(ColorTool.getColorByName(MOVE_BUTTON_COLOR));

        vocabularyLabel = new JLabel();
        successPercentLabel = new JLabel();

        testViewListSheetButtonsPanel = new JPanel();
        switchingContentButtonPanel = new JPanel();
        cardLayout = new CardLayout();
        perhapsShortenedVocabularyDescription = buildPerhapsShortenedVocabularyDescription();
        vocabularyAndPercentPanel = new JPanel();
        ownListsExtraButtonsPanel = new JPanel();

        shownButtonsType = VocabularyBarShownButtonsType.TEST_VIEW_LIST_SHEET_BUTTONS;

        setLabelText();
        initSuccessPercentLabel();
        initViewTestListSheet();
        initMoveUpAndDown();
        initOwnListsExtraButtons();
        initSwitchingContentButtonPanel();
        initOtherElements();
        fillPanel();
        setCorrectForegroundColor();
        fillVocabularyAndPercentPanel();
    }

    private void initSuccessPercentLabel() {
        GuiTools.setMonospacedFont(successPercentLabel);
        successPercentLabel.setBorder(new EmptyBorder(0, 5, 0, 5));
    }

    private String buildPerhapsShortenedVocabularyDescription() {
        String description = vocabulary.getDescription();
        if (options.isHideStartOfVocabularyDescription()) {
            if (options.isHideLessStartOfVocabularyDescriptionForVhs()
                    && description.contains("VHS")) {
                description = VocabularyTools.hideLessStartOfVocabularyDescriptionForVhs(
                        description);
            }
            else {
                description = VocabularyTools.hideStartOfVocabularyDescription(description);
            }
        }
        return description;
    }

    private void setLabelText() {
        vocabularyLabel.setText(perhapsShortenedVocabularyDescription);
    }

    private void initViewTestListSheet() {
        testViewListSheetButtonsPanel.setLayout(new FlowLayout());
        populateViewTestListSheet();
    }

    private void populateViewTestListSheet() {
        testViewListSheetButtonsPanel.removeAll();

        testViewListSheetButtonsPanel.add(createTestButton());
        testViewListSheetButtonsPanel.add(createViewButton());
        testViewListSheetButtonsPanel.add(createListButton());
        testViewListSheetButtonsPanel.add(createSheetButton());

        testViewListSheetButtonsPanel.repaint();
        testViewListSheetButtonsPanel.validate();
        testViewListSheetButtonsPanel.invalidate();
    }

    private Component createViewButton() {
        JButton button = VocabularyTrainerUiTools.createPicturedButton(
                IconDefinitions.VIEW_VOCABULARY, e -> viewVocabulary(), "Vokabeln anzeigen");
        if (vocabulary.getVocables().isEmpty()) {
            button.setEnabled(false);
        }
        return button;
    }

    private void viewVocabulary() {
        CompleteVocabularyViewerLogic viewer = new CompleteVocabularyViewerLogic(options, requester,
                vocabulary.getVocables(), "Gesamtanzeige '" + vocabulary.getDescription() + "'",
                (LongTimeProcessInformer) gui, gui.getLocation(), gui.getProgramImage());
        viewer.view();
        gui.setMessageLater("Das Vokabular \"" + perhapsShortenedVocabularyDescription
                + "\" wurde in Einzeldarstellung angezeigt.");
    }

    private Component createTestButton() {
        String pictureIdentifier;
        TranslationDirection direction = options.getTranslationDirection();
        if (direction == TranslationDirection.JAPANESE_TO_GERMAN) {
            pictureIdentifier = IconDefinitions.TEST_VOCABULARY_JAPANESE_TO_GERMAN;
        }
        else if (direction == TranslationDirection.GERMAN_TO_JAPANESE) {
            pictureIdentifier = IconDefinitions.TEST_VOCABULARY_GERMAN_TO_JAPANESE;
        }
        else {
            throw new RuntimeException("Unbekannte TranslationDirection " + direction + ".");
        }

        JButton button = VocabularyTrainerUiTools.createPicturedButton(
                pictureIdentifier, e -> testVocabulary(), "Vokabeln abfragen");
        if (vocabulary.getVocables().isEmpty()) {
            button.setEnabled(false);
        }
        return button;
    }

    private void testVocabulary() {
        VocableListTesterLogic tester = new VocableListTesterLogic(logic, gui,
                vocabulary.getVocables(), "Gesamtabfrage '" + vocabulary.getDescription() + "'");
        tester.test();

        setCorrectForegroundColor();
        gui.setMessageLater(
                "Das Vokabular \"" + perhapsShortenedVocabularyDescription + "\" wurde abgefragt.");
    }

    private Component createListButton() {
        JButton button = VocabularyTrainerUiTools.createPicturedButton(
                IconDefinitions.SHOW_VOCABULARY_LIST, e -> listVocabulary(),
                "Vokabeln als Liste anzeigen");
        if (vocabulary.getVocables().isEmpty()) {
            button.setEnabled(false);
        }
        return button;
    }

    private void listVocabulary() {
        VocabularyListerDialog dialog = new VocabularyListerDialog(options, requester,
                vocabulary.getVocables(), vocabulary.getDescription(),
                (LongTimeProcessInformer) gui, gui.getLocation(), gui.getProgramImage());
        dialog.setVisible(true);
        gui.setMessageLater("Das Vokabular \"" + perhapsShortenedVocabularyDescription
                + "\" wurde als Liste angezeigt.");
    }

    private Component createSheetButton() {
        JButton button = VocabularyTrainerUiTools.createPicturedButton(
                IconDefinitions.SHOW_VOCABULARY_SHEET, e -> listVocabularyAsSheet(),
                "Vokabeln auf einem Blatt anzeigen");
        if (vocabulary.getVocables().isEmpty()) {
            button.setEnabled(false);
        }
        return button;
    }

    private void listVocabularyAsSheet() {
        gui.startLongTimeProcess("Öffne Blattdarstellung mit " + vocabulary.getVocables().size()
                + " Vokabeln");
        new Thread(() -> listVocabularyAsSheetInOwnThread()).start();
    }

    private void listVocabularyAsSheetInOwnThread() {
        VocabularySheetDialog dialog = new VocabularySheetDialog(options, requester,
                vocabulary.getVocables(), vocabulary.getDescription(),
                (LongTimeProcessInformer) gui,
                gui.getLocation(),
                gui.getProgramImage());
        listVocabularyAsSheetInEdt(dialog);
    }

    private void listVocabularyAsSheetInEdt(VocabularySheetDialog dialog) {
        dialog.setVisible(true);
        dialog.requestFocus();
        gui.endLongTimeProcess();
        gui.setMessageLater("Das Vokabular \"" + perhapsShortenedVocabularyDescription
                + "\" wurde als Blatt angezeigt.");
    }

    private void initMoveUpAndDown() {
        moveUpAndDownButtonsPanel.setLayout(new GridLayout(1, 0, 2, 2));

        moveUpAndDownButtonsPanel.add(firstButton);
        moveUpAndDownButtonsPanel.add(upButton);
        moveUpAndDownButtonsPanel.add(downButton);
        moveUpAndDownButtonsPanel.add(lastButton);

        firstButton.setHorizontal(false);
        firstButton.forceSquare();
        firstButton.setPreferredSize(BUTTON_DIMENSION);
        firstButton.addActionListener(e -> moveBarToFirst());

        upButton.forceSquare();
        upButton.setPreferredSize(BUTTON_DIMENSION);
        upButton.addActionListener(e -> moveBarUp());

        downButton.forceSquare();
        downButton.setPreferredSize(BUTTON_DIMENSION);
        downButton.addActionListener(e -> moveBarDown());

        lastButton.setHorizontal(false);
        lastButton.forceSquare();
        lastButton.setPreferredSize(BUTTON_DIMENSION);
        lastButton.addActionListener(e -> moveBarToLast());
    }

    private void moveBarToFirst() {
        movingGui.moveBarToFirst(this);
    }

    private void moveBarUp() {
        movingGui.moveBarUp(this);
    }

    private void moveBarDown() {
        movingGui.moveBarDown(this);
    }

    private void moveBarToLast() {
        movingGui.moveBarToLast(this);
    }

    public void setUpAndDownButtonColorsAndEnabled() {
        boolean canMoveUp = movingGui.canBarMoveUp(this);
        Color upButtonColor = canMoveUp
                ? ColorTool.getColorByName(MOVE_BUTTON_COLOR)
                : ColorTool.getColorByName(DISABLED_COLOR);
        upButton.setForegroundColor(upButtonColor);
        upButton.setEnabled(canMoveUp);
        upButton.repaint();

        boolean canMoveDown = movingGui.canBarMoveDown(this);
        Color downButtonColor = canMoveDown
                ? ColorTool.getColorByName(MOVE_BUTTON_COLOR)
                : ColorTool.getColorByName(DISABLED_COLOR);
        downButton.setForegroundColor(downButtonColor);
        downButton.setEnabled(canMoveDown);
        downButton.repaint();
    }

    private void initOwnListsExtraButtons() {
        ownListsExtraButtonsPanel.setLayout(new GridLayout(1, 0, 2, 2));

        ownListsExtraButtonsPanel.add(createDeleteButton());
        ownListsExtraButtonsPanel.add(createEditButton());
        ownListsExtraButtonsPanel.add(createExportButton());
        ownListsExtraButtonsPanel.add(new JLabel());
    }

    private Component createDeleteButton() {
        JButton button = VocabularyTrainerUiTools.createPicturedButton(
                IconDefinitions.DELETE_OWN_LIST,
                e -> gui.deleteOwnList(getVocabularyAsOwnList()),
                "Vokabelliste löschen");
        button.setForeground(Color.RED);
        return button;
    }

    private Component createEditButton() {
        JButton button = VocabularyTrainerUiTools.createPicturedButton(
                IconDefinitions.EDIT_OWN_LIST,
                e -> gui.editOwnList(getVocabularyAsOwnList()),
                "Vokabelliste bearbeiten");
        return button;
    }

    private Component createExportButton() {
        JButton button = VocabularyTrainerUiTools.createPicturedButton(
                IconDefinitions.EXPORT_OWN_LIST,
                e -> gui.exportListOwnList(getVocabularyAsOwnList()),
                "Vokabelliste exportieren");
        return button;
    }

    private void initSwitchingContentButtonPanel() {
        switchingContentButtonPanel.setLayout(cardLayout);
        switchingContentButtonPanel.add(testViewListSheetButtonsPanel,
                TEST_VIEW_LIST_SHEET_BUTTONS_NAME);
        switchingContentButtonPanel.add(moveUpAndDownButtonsPanel, MOVE_UP_AND_DOWN_BUTTONS_NAME);
        switchingContentButtonPanel.add(ownListsExtraButtonsPanel, OWN_LISTS_EXTRA_BUTTONS_NAME);
    }

    private void initOtherElements() {
        GuiTools.biggerFont(vocabularyLabel, 7);
        //vocabularyLabel.setHorizontalTextPosition(JLabel.CENTER); // bringt nichts.
        vocabularyLabel.setHorizontalAlignment(JLabel.CENTER);
    }

    private void fillPanel() {
        panel.setLayout(new BorderLayout());
        GuiTools.createTitle(panel);

        panel.add(vocabularyAndPercentPanel, BorderLayout.CENTER);
        panel.add(createButtonBar(), BorderLayout.EAST);
    }

    private void fillVocabularyAndPercentPanel() {
        vocabularyAndPercentPanel.setLayout(new BorderLayout());

        vocabularyAndPercentPanel.removeAll();

        vocabularyAndPercentPanel.add(vocabularyLabel, BorderLayout.CENTER);
        if (options.isShowSuccessPercentInVocabularyBar()) {
            vocabularyAndPercentPanel.add(createSuccessPercentLabelPart(), BorderLayout.EAST);
        }
    }

    private Component createSuccessPercentLabelPart() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());

        panel.add(successPercentLabel, BorderLayout.CENTER);

        return panel;
    }

    private Component createButtonBar() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());

        panel.add(switchingContentButtonPanel, BorderLayout.CENTER);
        panel.add(selectionCheckBox, BorderLayout.EAST);

        return panel;
    }

    /**
     * Setzt die Vordergrundfarbe auf den nach den Optionen und ggf. dem Durchschnitt der
     * erfolgreichen Abfrage der Vokabeln aus dem Vokabular richtig.
     */
    public void setCorrectForegroundColor() {
        VocabularyTestSuccesssCalculator calculator =
                new VocabularyTestSuccesssCalculator(vocabulary, options, requester);
        calculator.calculate();

        Color foregroundColor = calculator.getForegroundColor();
        vocabularyLabel.setForeground(foregroundColor);
        vocabularyLabel.repaint();
        vocabularyLabel.invalidate();
        vocabularyLabel.validate();
        successPercentLabel.setText(calculator.getPercentText());
    }

    /** Aktualisiert den angezeigten Text je nach den Optionen. */
    public void setCorrectText() {
        perhapsShortenedVocabularyDescription = buildPerhapsShortenedVocabularyDescription();
        setLabelText();
        vocabularyLabel.repaint();
        vocabularyLabel.invalidate();
        vocabularyLabel.validate();
    }

    /** Aktualisiert den angezeigten Text zum Prozent des Erfolges je nach den Optionen. */
    public void showOrHidePercent() {
        fillVocabularyAndPercentPanel();
    }

    /** Getter für den Panel auf dem das Vokabular dargestellt wird. */
    public Component getPanel() {
        return panel;
    }

    /** Gibt an, ob die Vokabeln dieses Vokabulars ausgewählt wurden. */
    public boolean isSelected() {
        return selectionCheckBox.isSelected();
    }

    /** Legt fest, ob die Vokabeln dieses Vokabulars ausgewählt wurden. */
    public void setSelected(boolean selected) {
        selectionCheckBox.setSelected(selected);
    }

    /** Getter für das Vokabular zu welchem die Bar angezeigt wird. */
    public Vocabulary getVocabulary() {
        return vocabulary;
    }

    /** Getter für die Kategorie zu der das Vokabular gehört. */
    public String getVocabularyCategory() {
        return vocabulary.getCategory();
    }

    /** Getter für die Unterkategorie zu der das Vokabular gehört. */
    public String getVocabularySubCategory() {
        return vocabulary.getSubCategory();
    }

    /**
     * Zeigt auf dem Übersetzen-Buttons aller Bars das richtige Icon für die Richtung der
     * Übersetzung an.
     */
    public void showTranslationDirectionOnBarButtons() {
        populateViewTestListSheet();
    }

    /**
     * Gibt die Vokabelliste als eigene Liste zurück - wenn diese eine eigene Liste ist.
     *
     * Anderenfalls wird eine Ausnahme geworfen.
     */
    public OwnList getVocabularyAsOwnList() {
        if (vocabulary instanceof OwnList ownList) {
            return ownList;
        }
        else {
            throw new RuntimeException("Es wird versucht, ein Vokabular als eigene Liste "
                    + "zurückzugeben, welches keine eigene Liste ist.");
        }
    }

    /** Schaltet die Buttons zum Verschieben nach oben oder unten an oder aus. */
    public void showOrHideIndividualVocabularySortModeMoveButtons() {
        VocabularyBarShownButtonsType switchedType =
                VocabularyBarShownButtonsType.MOVE_UP_AND_DOWN_BUTTONS;
        changeSwitchingContentButtons(switchedType);
    }

    /**
     * Schaltet die Buttons zum Exportieren, Bearbeiten und Löschen von eigene Listen an oder aus.
     */
    public void toggleShowOwnListButtons() {
        VocabularyBarShownButtonsType switchedType =
                VocabularyBarShownButtonsType.OWN_LISTS_EXTRA_BUTTONS;
        changeSwitchingContentButtons(switchedType);
    }

    private void changeSwitchingContentButtons(VocabularyBarShownButtonsType switchedType) {
        if (shownButtonsType == switchedType) {
            showTestViewListSheetButtons();
        }
        else if (switchedType == VocabularyBarShownButtonsType.MOVE_UP_AND_DOWN_BUTTONS) {
            showMoveUpAndDownButtons();
        }
        else if (switchedType == VocabularyBarShownButtonsType.OWN_LISTS_EXTRA_BUTTONS) {
            showOwnListExtraButtons();
        }
        else {
            throw new RuntimeException("Logikfehler.");
        }
    }

    /** Zeigt die normalen Button zum Abfragen und betrachten der Vokabularien an. */
    public void showTestViewListSheetButtons() {
        shownButtonsType = VocabularyBarShownButtonsType.TEST_VIEW_LIST_SHEET_BUTTONS;
        String contentName = TEST_VIEW_LIST_SHEET_BUTTONS_NAME;
        switchCardLayout(contentName);
    }

    private void showMoveUpAndDownButtons() {
        shownButtonsType = VocabularyBarShownButtonsType.MOVE_UP_AND_DOWN_BUTTONS;
        String contentName = MOVE_UP_AND_DOWN_BUTTONS_NAME;
        switchCardLayout(contentName);
    }

    /** Zeigt bei eigenen Listen die Buttons zum Löschen, Bearbeiten und Exportieren an. */
    public void showOwnListExtraButtons() {
        shownButtonsType = VocabularyBarShownButtonsType.OWN_LISTS_EXTRA_BUTTONS;
        String contentName = OWN_LISTS_EXTRA_BUTTONS_NAME;
        switchCardLayout(contentName);
    }

    private void switchCardLayout(String contentName) {
        cardLayout.show(switchingContentButtonPanel, contentName);
        panel.repaint();
    }

    /**
     * Gibt an, ob diese VocabularyBar eine eigene Liste darstellt (wahr), oder (nur) ein normales
     * Vokabular (falsch).
     */
    public boolean isOwnListsBar() {
        return vocabulary instanceof OwnList;
    }

}
