package de.duehl.vocabulary.japanese.ui.dialog;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Image;
import java.awt.Point;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;

import de.duehl.swing.logic.LongTimeProcessInformer;
import de.duehl.swing.ui.GuiTools;
import de.duehl.swing.ui.dialogs.base.NonModalFrameDialogBase;
import de.duehl.swing.ui.elements.navigator.list.ListNavigator;
import de.duehl.vocabulary.japanese.data.Vocable;
import de.duehl.vocabulary.japanese.data.VocablesShuffleType;
import de.duehl.vocabulary.japanese.logic.internal.InternalDataRequester;
import de.duehl.vocabulary.japanese.logic.internal.data.InternalAdditionalVocableData;
import de.duehl.vocabulary.japanese.logic.persistence.Options;
import de.duehl.vocabulary.japanese.tools.VocableListShuffler;
import de.duehl.vocabulary.japanese.ui.components.VocableViewer;
import de.duehl.vocabulary.japanese.ui.components.button.OtherViewButtonPanel;
import de.duehl.vocabulary.japanese.ui.components.button.ViewButtonPressedReactor;
import de.duehl.vocabulary.japanese.ui.tools.VocabularyTrainerUiTools;

/**
 * Diese Klasse zeigt alle Vokabeln eines Vokabulars an.
 *
 * @version 1.01     2024-10-03
 * @author Christian Dühl
 */

public class CompleteVocabularyViewerDialog extends NonModalFrameDialogBase
        implements ViewButtonPressedReactor {

    /** Die Liste mit den abzufragenden Vokabeln. */
    private final List<Vocable> vocables;

    /** Die Programmoptionen. */
    private final Options options;

    /** Das Objekt das zu einer Vokabel die internen, benutzerabhängigen Daten abrufen kann. */
    private final InternalDataRequester requester;

    /** Der Navigator für die Vokabeln. */
    private final ListNavigator<Vocable> navigator;

    private final VocableViewer vocableViewer;

    /**
     * Zeigt drei Buttons an, mit denen man eine Menge von Vokabeln in Form einer
     * Einzeldarstellung, in Form einer Liste und in Form eines Vokabelblattes anzeigen lassen
     * kann.
     */
    private final OtherViewButtonPanel otherViewPanel;

    /** Der Button mit dem man das Einfärben des Hintergrunds ein- und ausschaltet. */
    private final JButton backgroundChangingButton;

    /**
     * Konstruktor.
     *
     * @param vocables
     *            Die Liste mit den abzufragenden Vokabeln.
     * @param description
     *            Die Beschreibung der Menge von Vokabeln.
     * @param requester
     *            Das Objekt das zu einer Vokabel die internen, benutzerabhängigen Daten abrufen
     *            kann.
     * @param options
     *            Die Programmoptionen.
     * @param informer
     *            Die Oberfläche, welche diese Klasse aufruft, auf der man eine GlassPane anzeigen
     *            kann.
     * @param parentLocation
     *            Position des Rahmens der Oberfläche, vor der dieser Dialog erzeugt wird.
     * @param programImage
     *            Anzuzeigendes ProgrammIcon.
     */
    public CompleteVocabularyViewerDialog(List<Vocable> vocables, String description,
            InternalDataRequester requester, Options options, LongTimeProcessInformer informer,
            Point parentLocation, Image programImage) {
        super(parentLocation, programImage,
                createDialogTitle(options, requester, vocables, description));
        addEscapeBehaviour();

        this.vocables = vocables;
        this.requester = requester;
        this.options = options;

        vocableViewer = new VocableViewer(options);
        navigator = new ListNavigator<>();

        otherViewPanel = new OtherViewButtonPanel(vocables, (ViewButtonPressedReactor) this,
                description, requester, options, informer, parentLocation, programImage);
        otherViewPanel.disableEinzeldarstellungButton();

        backgroundChangingButton = new JButton();

        init();
        fillDialog();
    }

    private static String createDialogTitle(Options options, InternalDataRequester requester,
            List<Vocable> vocables, String description) {
        return VocabularyTrainerUiTools.generateTitelWithVocabularyTestSuccesss(options, requester,
                vocables, description);
    }

    private void init() {
        initNavigator();
        initKeyBindings();
        initBackgroundChangingButton();
    }

    private void initNavigator() {
        navigator.setShowMethod(vocable -> showVocable(vocable));
        navigator.setDataList(determineVocablesToUse(vocables, options));
    }

    private void showVocable(Vocable vocable) {
        InternalAdditionalVocableData data = requester.getInternalDataForVocable(vocable);
        vocableViewer.showVocable(vocable, data);
        pack();
    }

    private List<Vocable> determineVocablesToUse(List<Vocable> vocables, Options options) {
        if (options.isUseTestOrderForViewingVocables()) {
            VocablesShuffleType type = options.getVocablesShuffleType();
            VocableListShuffler shuffler = new VocableListShuffler(options, vocables, type,
                    requester);
            shuffler.shuffle();
            return shuffler.getVocables();
        }
        else {
            return vocables;
        }
    }

    private void initKeyBindings() {
        setKeyBindingPageUp(() -> navigator.doClickOnPreviousButton());
        setKeyBindingPageDown(() -> navigator.doClickOnNextButton());

        // bringt es alles nicht ... Hmmmm....
        setKeyBindingLeft(() -> navigator.doClickOnPreviousButton());
        setKeyBindingRight(() -> navigator.doClickOnNextButton());
        setKeyBindingCtrlLeft(() -> navigator.doClickOnPreviousButton());
        setKeyBindingCtrlRight(() -> navigator.doClickOnNextButton());
        setKeyBindingAltLeft(() -> navigator.doClickOnPreviousButton());
        setKeyBindingAltRight(() -> navigator.doClickOnNextButton());

        setKeyBindingF1(() -> navigator.doClickOnPreviousButton());
        setKeyBindingF2(() -> navigator.doClickOnNextButton());
    }

    private void initBackgroundChangingButton() {
        setBackgroundChangingButtonTitle();
        backgroundChangingButton.addActionListener(e -> toggleBackground());
    }

    private void setBackgroundChangingButtonTitle() {
        String buttonText = ""
                + "Hintergrund "
                + (options.isColorVocableDependingOnLastSuccessWhenViewingVocable()
                        ? "aus"
                        : "ein")
                + "schalten";
        backgroundChangingButton.setText(buttonText);
    }

    private void toggleBackground() {
        boolean onOrOff = options.isColorVocableDependingOnLastSuccessWhenViewingVocable();
        onOrOff = !onOrOff;
        options.setColorVocableDependingOnLastSuccessWhenViewingVocable(onOrOff);
        setBackgroundChangingButtonTitle();

        Vocable vocable = navigator.getActualDataSet();
        showVocable(vocable);
    }

    /** Baut die Gui auf. */
    @Override
    protected void populateDialog() {
        add(createCenterPart(), BorderLayout.CENTER);
        add(createButtonsPart(),  BorderLayout.SOUTH);
    }

    private Component createCenterPart() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());

        panel.add(createCenteredNavigatorPanel(), BorderLayout.NORTH);
        panel.add(createVocabularyPart(), BorderLayout.CENTER);

        return panel;
    }

    private Component createCenteredNavigatorPanel() {
        JComponent centeredNavigation = GuiTools.center(navigator.getPanel());
        //GuiTools.createTitle(centeredNavigation);
        return centeredNavigation;
    }

    private Component createVocabularyPart() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());

        panel.add(vocableViewer.getPanel(), BorderLayout.CENTER);
        if (options.isShowDetailsButtonInEinzeldarstellung()) {
            panel.add(createDetailsButton(), BorderLayout.SOUTH);
        }

        return panel;
    }

    private Component createDetailsButton() {
        JButton button = new JButton("Details anzeigen");
        button.addActionListener(e -> showDetails());
        return button;
    }

    private void showDetails() {
        Vocable vocable = navigator.getActualDataSet();
        VocableWithInternaViewer viewer = new VocableWithInternaViewer(options, requester, vocable,
                getLocation(), getProgramImage());
        viewer.setVisible(true);
    }

    private Component createButtonsPart() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());


        panel.add(backgroundChangingButton, BorderLayout.WEST);
        panel.add(GuiTools.centerHorizontal(otherViewPanel.getPanel()), BorderLayout.CENTER);
        panel.add(createQuitButton(), BorderLayout.EAST);

        return panel;
    }

    private Component createQuitButton() {
        JButton button = new JButton("Beenden");
        button.addActionListener(e -> quit());
        return button;
    }

    private void quit() {
        closeDialog();
    }

    /** Wird aufgerufen, wenn die Vokabeln in einer anderen Ansicht angezeigt werden. */
    @Override
    public void otherViewOpened() {
        closeDialog();
    }

}
