package de.duehl.vocabulary.japanese.ui.dialog.testing;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Image;
import java.awt.Point;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;

import de.duehl.basics.text.NumberString;
import de.duehl.swing.ui.GuiTools;
import de.duehl.swing.ui.dialogs.base.ModalDialogBase;
import de.duehl.swing.ui.layout.VerticalLayout;
import de.duehl.vocabulary.japanese.logic.color.VocableColors;
import de.duehl.vocabulary.japanese.logic.persistence.Options;

/**
 * Diese Klasse zeigt den Erfolg oder Misserfolg nach dem Übersetzen aller abgefragten Variablen
 * an.
 *
 * @version 1.01     2024-07-30
 * @author Christian Dühl
 */

public class AllTranslationsEvaluationDialog extends ModalDialogBase {

    /** Die anzuzeigende Nachricht. */
    private final String message;

    /** Die Hintergrundfarbe. */
    private final Color backgroundColor;

    /** Der Button zum Beenden des Dialogs. */
    private final JButton quitButton;

    /** Der Titel mit Angabe der Prozentzahlen des Dialogs zur Abfrage der Vokabeln. */
    private final String originalDialogTitle;

    /**
     * Die Prozentzahl des Erfolgs der letzten zehn Abfragen dieser Vokabeln bevor der aktuelle
     * Test gelaufen war.
     */
    private final double lastTenTestsPercentBefore;

    /**
     * Die Prozentzahl des Erfolgs der letzten zehn Abfragen dieser Vokabeln nachdem der aktuelle
     * Test gelaufen ist.
     */
    private final double lastTenTestsPercentAfter;

    /** Die Prozentzahl des Erfolgs der gerade durchgeführten Abfragen dieser Vokabeln. */
    private final double actualPercent;

    /**
     * Die Prozentzahl mit zwei Nachkommastellen des Erfolgs der gerade durchgeführten Abfragen
     * dieser Vokabeln.
     */
    private String actualPercentTextWithTwoDigitsAfterComma;

    /**
     * Konstruktor.
     *
     * @param options
     *            Die Programmoptionen.
     * @param total
     *            Die Anzahl der übersetzten Vokabeln.
     * @param correct
     *            Die Anzahl der richtig übersetzten Vokabeln.
     * @param parentLocation
     *            Position des Rahmens der Oberfläche, vor der dieser Dialog erzeugt wird.
     * @param programImage
     *            Anzuzeigendes ProgrammIcon.
     * @param lastTenTestsPercentAfter
     *            Die Prozentzahl des Erfolgs der letzten zehn Abfragen dieser Vokabeln nachdem der
     *            aktuelle Test gelaufen ist.
     * @param lastTenTestsPercentBefore
     *            Die Prozentzahl des Erfolgs der letzten zehn Abfragen dieser Vokabeln bevor der
     *            aktuelle Test gelaufen war.
     * @param originalDialogTitle
     *            Der Titel mit Angabe der Prozentzahlen des Dialogs zur Abfrage der Vokabeln.
     */
    public AllTranslationsEvaluationDialog(Options options, int total, int correct,
            Point parentLocation, Image programImage, double lastTenTestsPercentBefore,
            double lastTenTestsPercentAfter, String originalDialogTitle) {
        super(parentLocation, programImage, "Auswertung der Übersetzungen");
        this.lastTenTestsPercentBefore = lastTenTestsPercentBefore;
        this.lastTenTestsPercentAfter = lastTenTestsPercentAfter;
        this.originalDialogTitle = originalDialogTitle;

        addEscapeBehaviour();

        message = createMessage(total, correct);

        VocableColors vocableColors = new VocableColors(options);
        backgroundColor = vocableColors.determineBackgroundColor(total, correct);
        actualPercent = NumberString.percentAsNumber(correct, total);
        actualPercentTextWithTwoDigitsAfterComma = NumberString.percent(correct, total);

        quitButton = new JButton("Beenden");

        fillDialog();
    }

    private String createMessage(int total, int correct) {
        String correctFromTotal = "Sie haben " + NumberString.taupu(correct) + " von "
                + NumberString.taupu(total) + " Vokabeln richtig übersetzt.";

        String successDescription;
        if (total == correct) {
            successDescription = "Sehr gut, alles richtig übersetzt!";
        }
        else {
            double percent = NumberString.percentAsNumber(correct, total);
            if (percent >= 90) {
                successDescription = "Das war schon sehr gut!";
            }
            else if (percent >= 75) {
                successDescription = "Ein gutes Ergebnis!";
            }
            else if (percent >= 50) {
                successDescription = "Immerhin war mindestens die Hälfte richtig.";
            }
            else if (percent >= 25) {
                successDescription = "Es wird, aber es gibt noch viel Luft nach oben.";
            }
            else {
                successDescription = "Üben, üben, üben ...";
            }
        }

        return correctFromTotal + "<br/>" + successDescription;
    }

    /** Baut die Gui auf. */
    @Override
    protected void populateDialog() {
        initElements();

        add(createCenterPart(), BorderLayout.CENTER);
        add(createButtonsPart(),  BorderLayout.SOUTH);

        pack();

        SwingUtilities.invokeLater(() -> quitButton.requestFocus());
    }

    private void initElements() {
        GuiTools.addReturnListener(quitButton, () -> quitButton.doClick());
    }

    private Component createCenterPart() {
        JPanel panel = new JPanel();
        panel.setLayout(new VerticalLayout(0, VerticalLayout.BOTH));

        panel.add(createMessagePart());
        panel.add(createTestedVocablesInfoPart());

        return new JScrollPane(panel);
    }

    private Component createTestedVocablesInfoPart() {
        String testedVocables = "Abgefragte Vokabeln: " + originalDialogTitle;
        String actualSuccessPercent = "Erfolg der aktuellen Abfrage in Prozent: "
                + actualPercentTextWithTwoDigitsAfterComma;
        String lastTenSuccessBefore = "Erfolg der letzten Zehn Abfragen VOR der aktuellen Abfrage "
                + "in Prozent: " + NumberString.twoDecimalPlaces(lastTenTestsPercentBefore);
        String lastTenSuccessAfter = "Erfolg der letzten Zehn Abfragen NACH der aktuellen Abfrage "
                + "in Prozent: " + NumberString.twoDecimalPlaces(lastTenTestsPercentAfter);

        String upOrDown;
        if (actualPercent > lastTenTestsPercentBefore) {
            upOrDown = "Sie haben sich verbessert.";
        }
        else if (actualPercent < lastTenTestsPercentBefore) {
            upOrDown = "Sie haben sich verschlechtert.";
        }
        else {
            upOrDown = "Sie waren genauso gut wie vorher.";
        }

        String labelText = "<html>"
                + "<br/>"
                + testedVocables
                + "<br/>"
                + actualSuccessPercent
                + "<br/>"
                + upOrDown
                + "<br/>"
                + "<br/>"
                + lastTenSuccessBefore
                + "<br/>"
                + lastTenSuccessAfter
                + "</html>";

        JLabel label = new JLabel(labelText);
        GuiTools.biggerFont(label, 5);
        label.setOpaque(true);
        label.setBackground(backgroundColor);
        return label;
    }

    private Component createMessagePart() {
        JLabel label = new JLabel("<html>" + message + "</html>");
        GuiTools.biggerFont(label, 5);
        label.setOpaque(true);
        label.setBackground(backgroundColor);
        return label;
    }

    private Component createButtonsPart() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());

        panel.add(createQuitButton(), BorderLayout.EAST);

        return panel;
    }

    private Component createQuitButton() {
        quitButton.addActionListener(e -> closeDialog());
        quitButton.setBackground(backgroundColor);
        return quitButton;
    }

}
