package de.duehl.vocabulary.japanese.ui.tools;

import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JButton;

import de.duehl.swing.ui.resources.IconLoader;
import de.duehl.vocabulary.japanese.data.Vocable;
import de.duehl.vocabulary.japanese.logic.internal.InternalDataRequester;
import de.duehl.vocabulary.japanese.logic.persistence.Options;
import de.duehl.vocabulary.japanese.logic.success.VocabularyTestSuccesssCalculator;
import de.duehl.vocabulary.japanese.ui.resources.IconDefinitions;

/**
 * Diese Klasse stellt Hilfsmethoden für die Gui zur Verfügung.
 *
 * @version 1.01     2024-01-20
 * @author Christian Dühl
 */

public class VocabularyTrainerUiTools {

    /**
     * Diese Methode erzeugt den Titel für einen Dialog, welcher auch den Erfolg in Prozent
     * anzeigt, der bei den letzten Abfragen der Vokabeln erzielt wurde.
     *
     * @param options
     *            Die Programmoptionen.
     * @param requester
     *            Das Objekt das zu einer Vokabel die internen, benutzerabhängigen Daten abrufen
     *            kann.
     * @param vocables
     *            Die Liste mit den abzufragenden Vokabeln.
     * @param frontTitle
     *            Der vordere Teil des Titels.
     * @return Der Titel mit Prozentangabe des Erfolgs bei den letzten Abfragen der Vokabeln in
     *         Klammern dahinter.
     */
    public static String generateTitelWithVocabularyTestSuccesss(Options options,
            InternalDataRequester requester, List<Vocable> vocables, String frontTitle) {
        VocabularyTestSuccesssCalculator calculator =
                new VocabularyTestSuccesssCalculator(vocables, options, requester);
        calculator.calculate();
        String percent = calculator.getPercentTextWithTwoDigitsAfterComma();

        return frontTitle + " (" + percent + "%)";
    }

    /**
     * Diese Methode erzeugt die Prozentzahl des Erfolgs, der bei den letzten Abfragen der Vokabeln
     * erzielt wurde.
     *
     * @param options
     *            Die Programmoptionen.
     * @param requester
     *            Das Objekt das zu einer Vokabel die internen, benutzerabhängigen Daten abrufen
     *            kann.
     * @param vocables
     *            Die Liste mit den abzufragenden Vokabeln.
     * @return Die Prozentzahl des Erfolgs bei den letzten Abfragen der Vokabeln.
     */
    public static double createLastTenTestsPercent(Options options, InternalDataRequester requester,
            List<Vocable> vocables) {
        VocabularyTestSuccesssCalculator calculator =
                new VocabularyTestSuccesssCalculator(vocables, options, requester);
        calculator.calculate();
        return calculator.getPercent();
    }

    /**
     * Erzeugt einen Button mit einem Icon.
     *
     * @param pictureIdentifier
     *            Bezeichner des Bildes, im Normalfall eine der öffentlichen Konstanten aus
     *            IconDefinitions.
     * @param listener
     *            Wird beim Betätigen des Buttons ausgeführt.
     * @param toolTipText
     *            Wird angezeigt, wenn die Maus eine Weile über dem Button verweilt.
     * @return Der erzeugte Button.
     */
    public static JButton createPicturedButton(String pictureIdentifier, ActionListener listener,
            String toolTipText) {
        JButton button = new JButton(loadIcon(pictureIdentifier));
        button.setBorder(BorderFactory.createRaisedBevelBorder());
        button.addActionListener(listener);
        button.setToolTipText(toolTipText);
        return button;
    }

    private static Icon loadIcon(String imageIdentifier) {
        IconDefinitions iconDefinitions = new IconDefinitions();
        IconLoader iconLoader = iconDefinitions.createIconLoader();
        return iconLoader.loadIcon(imageIdentifier);
    }

}
