package de.duehl.vocabulary.japanese.website.update.ownlists;

import java.util.ArrayList;
import java.util.List;

import de.duehl.basics.io.FileHelper;
import de.duehl.basics.text.handling.BooleanHandling;
import de.duehl.swing.ui.GuiTools;
import de.duehl.vocabulary.japanese.logic.VocabularyTrainerLogic;
import de.duehl.vocabulary.japanese.ui.VocabularyTrainerGui;
import de.duehl.vocabulary.japanese.website.update.ownlists.data.NewestOwnListVersionListEntry;
import de.duehl.vocabulary.japanese.website.update.ownlists.data.OwnListInterestAndVersionEntry;
import de.duehl.vocabulary.japanese.website.update.ownlists.io.OwnListVersionAndInterestFile;
import de.duehl.vocabulary.japanese.website.update.ownlists.parser.NewestOwnListsVersionParser;
import de.duehl.vocabulary.japanese.website.update.ownlists.parser.OwnListVersionAndInterestParser;

import static de.duehl.vocabulary.japanese.website.VocaluaryTrainerWebsiteConstants.*;
import static de.duehl.vocabulary.japanese.website.download.WebsiteFileContentDownloader.*;
import static de.duehl.vocabulary.japanese.website.update.UpdateTools.*;

/**
 * Diese Klasse kümmert sich um Updates der bereitgestellten Gruppen von eigenen Listen.
 *
 * @version 1.01     2024-10-06
 * @author Christian Dühl
 */

public class OwnListGroupsUpdater {

    private static final OwnListInterestAndVersionEntry NOT_FOUND_INTERST_AND_VERSION =
            new OwnListInterestAndVersionEntry("####NICHT GEFUNDEN###", false, -1);


    /** Die Logik des Vokabel-Trainers. */
    @SuppressWarnings("unused")
    private final VocabularyTrainerLogic logic;

    /** Die grafische Oberfläche des Vokabel-Trainers. */
    private final VocabularyTrainerGui gui;

    /**
     * Gibt an, dass die Prüfungen auf neue Versionen beim Start des Programms stattfinden.
     * Anderenfalls hat der Benutzer sie über das Update-Menü angestoßen.
     */
    private final boolean checkingNewVersionsOnStartUp;

    /** Die Liste der Einträge in der Datei Neuste_Version_Listen.txt auf dem Server. */
    private List<NewestOwnListVersionListEntry> newestOwnListVersionEntries;

    /**
     * Die Liste der Einträge in der Datei Eigene_Listen_Interessen_und_Versionen.txt auf dem
     * Rechner des Benutzers.
     */
    private List<OwnListInterestAndVersionEntry> interestAndVersionEntries;

    /**
     * Die Liste derjenigen Gruppennamen, die der Benutzer bisher noch nicht oder nur in einer
     * älteren Version kannte.
     */
    private List<String> newerOrUnseenGroupNames;

    /**
     * Konstruktor.
     *
     * @param logic
     *            Die Logik des Vokabel-Trainers.
     * @param gui
     *            Die grafische Oberfläche des Vokabel-Trainers.
     * @param checkingNewVersionsOnStartUp
     *            Gibt an, dass die Prüfungen auf neue Versionen beim Start des Programms
     *            stattfinden. Anderenfalls hat der Benutzer sie über das Update-Menü angestoßen.
     */
    public OwnListGroupsUpdater(VocabularyTrainerLogic logic, VocabularyTrainerGui gui,
            boolean checkingNewVersionsOnStartUp) {
        this.logic = logic;
        this.gui = gui;
        this.checkingNewVersionsOnStartUp = checkingNewVersionsOnStartUp;
    }

    /** Prüft auf eine aktuellere Version der eigenen Listen. */
    public void update() {
        newestOwnListVersionEntries = downloadNewestOwnListVersionListEntriesFromServer();

        if (newestOwnListVersionEntries.isEmpty()) {
            if (!checkingNewVersionsOnStartUp) {
                informUserAboutDownloadFailure("der eigenen Listen", gui);
            }
        }
        else {
            checkOwnListVersionsAgainstWebsiteVersions();
        }
    }

    private List<NewestOwnListVersionListEntry> downloadNewestOwnListVersionListEntriesFromServer() {
        String content = downloadFileViaUrlFromServer(NEWEST_OWN_LISTS_VERSION_ON_SERVER_URL);
        if (content.equals(NO_DOWNLOAD_SUCCESS)) {
            return new ArrayList<>();
        }

        NewestOwnListsVersionParser parser = new NewestOwnListsVersionParser(content);
        parser.parse();
        if (parser.isSuccess()) {
            return parser.getNewestOwnListVersionListEntries();
        }
        return new ArrayList<>();
    }

    private void checkOwnListVersionsAgainstWebsiteVersions() {
        interestAndVersionEntries = loadOwnListVersionAndInterests();
        newerOrUnseenGroupNames = determineNewerVersionsOrUnknownGroupsOnTheServer();

        if (newerOrUnseenGroupNames.isEmpty()) {
            informUserAboutOwnListsArtActual();
        }
        else {
            updateNewerOwnListVersionAndInterestEntries();
            storeVersionAndInterestFile();

            if (isUserInterestedInChanges()) {
                showOwnListInterestSelectionAndDownloadDialog();
            }
        }
    }

    /** Lädt die Datei "Eigene_Listen_Interessen_und_Versionen.txt" vom Rechner des Benutzer.  */
    private List<OwnListInterestAndVersionEntry> loadOwnListVersionAndInterests() {
        String filename = OwnListVersionAndInterestFile.OWN_LIST_INTERSTS_AND_VERSIONS_FILENAME;
        if (!FileHelper.isFile(filename)) {
            return new ArrayList<>();
        }

        String content = FileHelper.readFileToString(filename);
        OwnListVersionAndInterestParser parser = new OwnListVersionAndInterestParser(content);
        parser.parse();
        if (parser.isSuccess()) {
            return parser.getOwnListInterestAndVersionEntries();
        }
        return new ArrayList<>();
    }

    /**
     * Ermittelt die Gruppennamen, die der Benutzer bislang nicht zu Gesicht bekommen hat oder bei
     * der die Gruppen auf dem Server eine höhere Version haben.
     */
    private List<String> determineNewerVersionsOrUnknownGroupsOnTheServer() {
        List<String> groupNames = new ArrayList<>();

        for (NewestOwnListVersionListEntry newestEntry : newestOwnListVersionEntries) {
            String groupName = newestEntry.getGroupName();

            OwnListInterestAndVersionEntry interestEntry =
                    findOwnListInterestAndVersionEntryByGroupname(groupName);

            if (NOT_FOUND_INTERST_AND_VERSION.equals(interestEntry)) {
                groupNames.add(groupName); // neuer Gruppenname, den der Benutzer bisher nicht kennt
            }
            else {
                int versionSeenByUser = interestEntry.getVersion();
                if (versionSeenByUser < newestEntry.getVersion()) {
                    groupNames.add(groupName); // ist auf dem Server neuer
                }
            }
        }

        return groupNames;
    }

    private OwnListInterestAndVersionEntry findOwnListInterestAndVersionEntryByGroupname(
            String groupName) {
        for (OwnListInterestAndVersionEntry interestEntry : interestAndVersionEntries) {
            if (interestEntry.getGroupName().equals(groupName)) {
                return interestEntry;
            }
        }

        return NOT_FOUND_INTERST_AND_VERSION;
    }

    private void informUserAboutOwnListsArtActual() {
        String title = "Es gibt keine Neuerungen bei den Gruppen mit eigenen Listen";
        String message = title;
        GuiTools.informUserInEdt(gui.getWindowAsComponent(), title, message);
    }

    /**
     * Aktualisiert die Einträge in Daten des Benutzers was bislang nicht gesehene Gruppen oder
     * höhere Versionen bereits gesehener Gruppen angeht.
     *
     * Hierbei orientiert man sich an den Einträgen vom Server.
     */
    private void updateNewerOwnListVersionAndInterestEntries() {
        List<OwnListInterestAndVersionEntry> updatedInterestAndVersionEntries = new ArrayList<>();

        for (NewestOwnListVersionListEntry newestEntry : newestOwnListVersionEntries) {
            String groupName = newestEntry.getGroupName();

            OwnListInterestAndVersionEntry interestEntry =
                    findOwnListInterestAndVersionEntryByGroupname(groupName);

            if (NOT_FOUND_INTERST_AND_VERSION.equals(interestEntry)) {
                interestEntry = new OwnListInterestAndVersionEntry(groupName, true,
                        newestEntry.getVersion());
            }
            else {
                interestEntry = new OwnListInterestAndVersionEntry(groupName,
                        interestEntry.isInterested(), newestEntry.getVersion());
            }
            updatedInterestAndVersionEntries.add(interestEntry);
        }

        interestAndVersionEntries = updatedInterestAndVersionEntries;
    }

    /**
     * Speichert die aktualisierte Datei "Eigene_Listen_Interessen_und_Versionen.txt" auf dem
     * Rechner des Benutzer.
     */
    private void storeVersionAndInterestFile() {
        String filename = OwnListVersionAndInterestFile.OWN_LIST_INTERSTS_AND_VERSIONS_FILENAME;

        List<String> lines = new ArrayList<>();
        for (OwnListInterestAndVersionEntry entry : interestAndVersionEntries) {
            String line = entry.getGroupName()
                    + " - "
                    + BooleanHandling.booleanToString(entry.isInterested())
                    + " - "
                    + entry.getVersion();
            lines.add(line);
        }

        FileHelper.writeLinesToFile(lines, filename);
    }

    /** Prüft, ob der Benutzer an mindestens einer der Gruppen interessiert ist. */
    private boolean isUserInterestedInChanges() {
        for (String groupName : newerOrUnseenGroupNames) {
            OwnListInterestAndVersionEntry interestEntry =
                    findOwnListInterestAndVersionEntryByGroupname(groupName);

            if (NOT_FOUND_INTERST_AND_VERSION.equals(interestEntry)) {
                // kann nicht passieren, da upgedatet wurde
            }
            else {
                if (interestEntry.isInterested()) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * Zeigt dem Benutzer den Dialog an, in der er die Einstellung seiner Interessen vornehmen und
     * eine oder mehrere Gruppen von Liste herunterladen kann.
     *
     *   +----------------+------------------+----------+---------------+------------+
     *   | Listengruppe   |  Listenversion   |   neu?   |    Interesse  |  Download  |
     *   +----------------+------------------+----------+---------------+------------+
     *   | FROM_ZERO_1    |  35              |   nein   |    nein       |    [ ]     |
     *   | FROM_ZERO_2    |  17              |   nein   |    nein       |    [ ]     |
     *   | FROM_ZERO_3    |  3               |   nein   |    nein       |    [ ]     |
     *   | SCHRITT_FUER_1 |  4               |   nein   |    nein       |    [ ]     |
     *   | MARUGOTO_A1    |  7               |   ja     |    ja         |    [x]     |
     *   | VHS_A1_1       |  11              |   nein   |    ja         |    [ ]     |
     *   | VHS_A1_2       |  1               |   ja     |    ja         |    [x]     |
     *   +----------------+------------------+----------+---------------+------------+
     *
     *   Angezeigt wird statt des Kürzels der Gruppe natürlich die im Enum hinterlegte
     *   Beschreibung. Vielleicht hinterlege ich da auch noch eine noch kürzere Version.
     *   Siehe de.duehl.vocabulary.japanese.logic.ownlists.OwnListGroup.
     *   In den Entries ist das Kürzel gespeichert, also ("FROM_ZERO_1").
     *
     *   Am Ende wird die getroffene Auswahl heruntergeladen, in das ggf. vom Benutzer
     *   festzulegende Listenverzeichnis (siehe Optionen, wenn Default, fragen)
     *   entpackt und der Listenimportdialog geöffnet. Oder importiere ich diese Listen
     *   dann einfach direkt? Ist vielleicht besser.
     */
    private void showOwnListInterestSelectionAndDownloadDialog() {
        String title = "Es gibt Neuerungen bei den Gruppen mit eigenen Listen.";
        String message = "Es gibt Neuerungen bei den Gruppen mit eigenen Listen, der eigentliche "
                + "Dialog ist aber noch in Arbeit.";
        GuiTools.informUserInEdt(gui.getWindowAsComponent(), title, message);
        /* TODO
         *
         * Hier muss ich später den passenden Dialog erstellen, in dem der Benutzer das Interesse
         * ändern und Gruppen zum Herunterladen auswählen kann.
         *
         * Dazu verwendet man die Infos aus interestAndVersionEntries und
         * newerOrUnseenGroupnames um Neuerungen zu markieren, was sinnvoll wäre.
         */


        /*
         * TODO
         *
         * Bei den eigenen Listen ist der Pfad auch in den Optionen, man sollte wohl aber auch
         * fragen, wohin es gespeichert werden soll und dazu den Pfad benutzen. Siehe Export von
         * eigenen Listen.
         *
         */
    }

    /*
     * TODO das muss alles gründlich getestet werden, sobald wirklich Listen heruntergeladen werden
     * können.
     */

}
