package de.duehl.vocabulary.japanese.website.update.program;

import de.duehl.basics.version.Version;
import de.duehl.swing.ui.GuiTools;
import de.duehl.vocabulary.japanese.VocabularyTrainerVersion;
import de.duehl.vocabulary.japanese.logic.VocabularyTrainerLogic;
import de.duehl.vocabulary.japanese.ui.VocabularyTrainerGui;
import de.duehl.vocabulary.japanese.ui.dialog.NewUpdateFoundDialog;

import static de.duehl.vocabulary.japanese.website.VocaluaryTrainerWebsiteConstants.*;
import static de.duehl.vocabulary.japanese.website.download.WebsiteFileContentDownloader.*;
import static de.duehl.vocabulary.japanese.website.update.UpdateTools.*;

/**
 * Diese Klasse kümmert sich um Updates des Programms.
 *
 * @version 1.01     2024-10-05
 * @author Christian Dühl
 */

public class ProgramUpdater {

    /** Die Logik des Vokabel-Trainers. */
    @SuppressWarnings("unused")
    private final VocabularyTrainerLogic logic;

    /** Die grafische Oberfläche des Vokabel-Trainers. */
    private final VocabularyTrainerGui gui;

    /**
     * Gibt an, dass die Prüfungen auf neue Versionen beim Start des Programms stattfinden.
     * Anderenfalls hat der Benutzer sie über das Update-Menü angestoßen.
     */
    private final boolean checkingNewVersionsOnStartUp;

    /**
     * Konstruktor.
     *
     * @param logic
     *            Die Logik des Vokabel-Trainers.
     * @param gui
     *            Die grafische Oberfläche des Vokabel-Trainers.
     * @param checkingNewVersionsOnStartUp
     *            Gibt an, dass die Prüfungen auf neue Versionen beim Start des Programms
     *            stattfinden. Anderenfalls hat der Benutzer sie über das Update-Menü angestoßen.
     */
    public ProgramUpdater(VocabularyTrainerLogic logic, VocabularyTrainerGui gui,
            boolean checkingNewVersionsOnStartUp) {
        this.logic = logic;
        this.gui = gui;
        this.checkingNewVersionsOnStartUp = checkingNewVersionsOnStartUp;
    }

    /** Prüft auf eine aktuellere Version des Programms. */
    public void update() {
        String versionNumberOnServer = downloadProgramVersionFromServer();
        if (versionNumberOnServer.equals(NO_DOWNLOAD_SUCCESS)) {
            if (!checkingNewVersionsOnStartUp) {
                informUserAboutDownloadFailure("des Programms", gui);
            }
        }
        else {
            String verionNumberRunning = determineProgramVersionRunning();
            Version versionOnServer = new Version(versionNumberOnServer, "interessiert nicht");
            Version verionRunning = new Version(verionNumberRunning, "interessiert nicht");
            if (versionOnServer.isNewerThan(verionRunning)) {
                informUserAboutNewerProgramVesion(verionNumberRunning, versionNumberOnServer);
            }
            else if (!checkingNewVersionsOnStartUp) {
                informUserAboutProgramVersionIsActual(verionNumberRunning);
            }
        }
    }


    private String downloadProgramVersionFromServer() {
        return downloadFileViaUrlFromServerAndGetContentOfFirstLine(
                NEWEST_PROGRAM_VERSION_ON_SERVER_URL);
    }

    private String determineProgramVersionRunning() {
        return VocabularyTrainerVersion.getOnlyVersion();
    }

    private void informUserAboutNewerProgramVesion(String verionNumberRunning,
            String versionNumberOnServer) {
        String title = "Es gibt eine neuere Programmversion";
        String message = "Es gibt eine neuere Programmversion!\n\n"
                + "Aktuelle Programmversion: " + verionNumberRunning + "\n"
                + "Neuere Programmversion: " + versionNumberOnServer;
        String beforeUrl = "Bitte besuchen sie die Seite";
        String url = SHORT_VOCBULARY_TRAINER_WEBPAGE_URL;
        String afterUrl = "und laden sie die neue Version herunter.";
        NewUpdateFoundDialog dialog = new NewUpdateFoundDialog(title, message, beforeUrl, url,
                afterUrl, gui.getLocation(), gui.getProgramImage());
        dialog.setVisible(true);
    }

    private void informUserAboutProgramVersionIsActual(String verionNumberRunning) {
        String title = "Der Vokabeltrainer ist aktuell";
        String message = "Die verwendete Programmversion " + verionNumberRunning + " ist aktuell.";
        GuiTools.informUserInEdt(gui.getWindowAsComponent(), title, message);
    }

    /*
     * TODO
     *
     * Download aktuellere Programm-Version:
     * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
     *
     * Ich zeige zunächst mal nur an, dass es eine neuere Version gibt und auf die (noch zu
     * schreibende) Webseite mit der Anleitung verweisen.
     *
     * Später wäre eine direkte Möglichkeit zum Download aus dem Programm gut. Man kann die Jar
     * dann neben die aktuell laufende kopieren.
     * Und ggf. analog zu Jarify bestehende Batches im gleichen Verzeichnis anpassen.
     *
     *
     * Sollte ich vielleicht ein "Installationsprogramm" anbieten, das eine solche Batchdatei
     * anlegt und dabei nach der installierten Java-Executable fragt?
     * Aber vermutlich verwirrt das die Leute eher. Sie installieren Java normal und führen dann
     * einfach die Jar-Datei aus.
     *
     * Ich sollte auf der Webseite erwähnen, dass die Jar ausführbar ist!
     *
     * --------------------------------------------------------------------------------------------
     *
     * - Notizen zum Herunterladen binärer Dateien:
     *   https://stackoverflow.com/questions/10960409/how-do-i-save-a-file-downloaded-with-httpclient-into-a-specific-folder
     *   https://coderanch.com/t/676274/java/download-binary-file-web-server
     *   Das brauche ich, wenn ich über http die neue Version der Vokabeln oder des Programms
     *   herunterladen will.
     */

}
